-- CREACI DE TAULES SENSE REFERENCIAR-- CPM 

-- LOGS
CREATE TABLE Logs (
id_log INTEGER CONSTRAINT PK_id_log PRIMARY KEY,
nom_procediment VARCHAR2(80) CONSTRAINT NN_nom_procediment NOT NULL,
param_entrada VARCHAR2(4000) CONSTRAINT NN_param_entrada NOT NULL,
param_sortida VARCHAR2(4000) CONSTRAINT NN_param_sortida NOT NULL,
resultat VARCHAR2(4000) CONSTRAINT NN_resultat NOT NULL,
usuari_BD VARCHAR2(50) CONSTRAINT NN_usuari_BD NOT NULL,
datahora TIMESTAMP CONSTRAINT NN_datahora NOT NULL
);

-- PAS
CREATE TABLE Pais (
codi_pais CHAR(2) CONSTRAINT PK_codi_pais PRIMARY KEY,
nom_pais VARCHAR2(50) CONSTRAINT NN_nom_pais NOT NULL
);

-- REGI
CREATE TABLE Regio (
id_regio INTEGER CONSTRAINT PK_id_regio PRIMARY KEY,
codi_pais CHAR(2) CONSTRAINT NN_regio_pais NOT NULL,
nom_regio VARCHAR2(80) CONSTRAINT NN_nom_regio NOT NULL
);

-- LOCALITAT
CREATE TABLE Localitat (
id_localitat INTEGER CONSTRAINT PK_id_localitat PRIMARY KEY,
id_regio INTEGER CONSTRAINT NN_localitat_regio NOT NULL,
nom_localitat VARCHAR2(80) CONSTRAINT NN_nom_localitat NOT NULL
);

-- CENS
CREATE TABLE Cens (
id_cens INTEGER CONSTRAINT PK_id_cens PRIMARY KEY,
tipus_cens CHAR(1) 	CONSTRAINT ck_cens_tipus_cens CHECK (tipus_cens IN ('G','E','R','L')) 
					CONSTRAINT NN_tipus_cens NOT NULL
);

-- CENS_GLOBAL
CREATE TABLE CensGlobal (
id_cens INTEGER CONSTRAINT PK_id_cens_global PRIMARY KEY,
edat_minima NUMBER(2) DEFAULT 18 CONSTRAINT NN_cg_edat_minima NOT NULL
);

-- CENS_LOCAL
CREATE TABLE CensLocal (
id_cens INTEGER CONSTRAINT PK_id_cens_local PRIMARY KEY,
id_localitat INTEGER CONSTRAINT NN_cl_id_localitat NOT NULL
);

-- CENS_REGIONAL
CREATE TABLE CensRegional (
id_cens INTEGER CONSTRAINT PK_id_cens_regional PRIMARY KEY,
id_regio INTEGER CONSTRAINT NN_cr_id_regio NOT NULL
);

-- CENS_ESTATAL
CREATE TABLE CensEstatal (
id_cens INTEGER CONSTRAINT PK_id_cens_estatal PRIMARY KEY,
codi_pais CHAR(2) CONSTRAINT NN_ce_codi_pais NOT NULL
);

-- VOTACI
CREATE TABLE Votacio (
id_votacio INTEGER CONSTRAINT PK_votacio PRIMARY KEY,
titol VARCHAR2(100) CONSTRAINT NN_votacio_titol NOT NULL,
descripcio VARCHAR2(100) CONSTRAINT NN_votacio_descripcio NOT NULL,
data_inicial TIMESTAMP CONSTRAINT NN_votacio_data_inicial NOT NULL,
data_final TIMESTAMP CONSTRAINT NN_votacio_data_final NOT NULL,
data_publicacio DATE CONSTRAINT NN_votacio_data_publicacio NOT NULL,
president VARCHAR2(60) CONSTRAINT NN_votacio_president NOT NULL,
estat CHAR(1) CONSTRAINT ck_votacio_estat CHECK 
(estat IN ('E','P','O','F')) CONSTRAINT NN_votacio_estat NOT NULL,
web VARCHAR2(100),
perc_participacio NUMBER(5,2) CONSTRAINT ck_perc_between_participacio CHECK 
(perc_participacio >= 0 and perc_participacio <= 100),
num_pers_capacitat_vot INTEGER DEFAULT 0,
vots_totals INTEGER DEFAULT 0,
id_opcio_guanyadora INTEGER,
perc_vots_opcio_guanyadora NUMBER(5,2) CONSTRAINT ck_perc_between_guanyadora CHECK 
(perc_vots_opcio_guanyadora >= 0 and perc_vots_opcio_guanyadora <= 100)
);

CREATE INDEX index_votacio
ON Votacio (data_inicial);

-- OPCIO
CREATE TABLE Opcio (
id_opcio INTEGER CONSTRAINT PK_id_opcio PRIMARY KEY,
descripcio VARCHAR2(100) CONSTRAINT NN_opcio_descripcio NOT NULL
);


-- OPCIO_VOTACIO
CREATE TABLE Opcio_Votacio (
id_opcio INTEGER CONSTRAINT NN_ov_id_opcio NOT NULL,
id_votacio INTEGER CONSTRAINT NN_ov_id_votacio NOT NULL,
num_vots_opcio INTEGER DEFAULT 0,
perc_sobre_total NUMBER(5,2) CONSTRAINT ck_perc_between_total CHECK 
(perc_sobre_total >= 0 and perc_sobre_total <= 100),
CONSTRAINT PK_Opcio_Votacio PRIMARY KEY 
(id_opcio, id_votacio)
);

-- VOTA
CREATE TABLE Vota (
codi_ciutada INTEGER CONSTRAINT NN_v_codi_ciutada NOT NULL,
id_votacio INTEGER CONSTRAINT NN_v_id_votacio NOT NULL,
id_opcio INTEGER CONSTRAINT NN_v_id_opcio NOT NULL,
data_hora_vot TIMESTAMP CONSTRAINT NN_v_data_hora NOT NULL,
CONSTRAINT PK_Vota PRIMARY KEY (codi_ciutada, id_votacio)
);

-- CIUTADA
CREATE TABLE Ciutada (
codi_ciutada INTEGER CONSTRAINT PK_c_codi_ciutada PRIMARY KEY,
id_localitat INTEGER CONSTRAINT NN_c_id_localitat NOT NULL,
data_naixament DATE CONSTRAINT NN_c_data_naixament NOT NULL,
data_alta DATE CONSTRAINT NN_c_data_alta NOT NULL,
data_baixa DATE
);

-- ASSIGNACIO
CREATE TABLE Assignacio (
id_votacio INTEGER CONSTRAINT NN_a_id_votacio NOT NULL,
codi_ciutada INTEGER CONSTRAINT NN_a_codi_ciutada NOT NULL,
id_cens INTEGER CONSTRAINT NN_a_id_cens NOT NULL,
data_assignacio DATE DEFAULT SYSDATE CONSTRAINT NN_a_data_assignacio NOT NULL,
CONSTRAINT PK_Assignacio PRIMARY KEY 
(id_votacio, codi_ciutada, id_cens)
);


-- E1  Nombre de votacions finalitzades x pas i any
CREATE TABLE E1 (
codi_pais CHAR(2) CONSTRAINT NN_E1_codi_pais NOT NULL,
any_votacio INTEGER CONSTRAINT NN_E1_any_votacio NOT NULL,
num_votacions NUMBER(10) CONSTRAINT NN_E1_num_votacions NOT NULL,
CONSTRAINT PK_E1 PRIMARY KEY (codi_pais, any_votacio)
);


-- E2 Valor mitj del percentatge de participaci x cens i any concret
CREATE TABLE E2 (
id_cens INTEGER CONSTRAINT NN_E2_id_cens NOT NULL,
any_votacio INTEGER CONSTRAINT NN_E2_any_votacio NOT NULL,
valor_mitja NUMBER(5,2) CONSTRAINT ck_valor_mitja CHECK 
(valor_mitja >= 0 and valor_mitja <= 100),
CONSTRAINT PK_E2 PRIMARY KEY (id_cens, any_votacio)
);


-- E3 Nombre de votacions x cens i any concret
CREATE TABLE E3 (
id_cens INTEGER CONSTRAINT NN_E3_id_cens NOT NULL,
any_votacio INTEGER CONSTRAINT NN_E3_any_votacio NOT NULL,
num_votacions NUMBER(10) CONSTRAINT NN_E3_num_votacions NOT NULL,
CONSTRAINT PK_E3 PRIMARY KEY (id_cens, any_votacio)
);


-- E4 Votaci amb ms participaci x any concret
CREATE TABLE E4 (
any_votacio INTEGER CONSTRAINT PK_E4_any_votacio PRIMARY KEY,
id_votacio INTEGER CONSTRAINT NN_E4_id_votacio NOT NULL,
participacio_e4 NUMBER(5,2) CONSTRAINT NN_E4_participacio NOT NULL
);


--E5 Localitat europea amb ms vots que han ems els seus ciutadans x any concret
CREATE TABLE E5 (
any_concret INTEGER CONSTRAINT PK_E5_any_concret PRIMARY KEY,
id_localitat INTEGER CONSTRAINT NN_E5_id_localitat NOT NULL,
num_votacions_e5 INTEGER CONSTRAINT NN_E5_num_votacions NOT NULL
);


--E6 Percentatge de ciutadans que no han votat per any concret
CREATE TABLE E6 (
any_concret INTEGER CONSTRAINT PK_E6_any_concret PRIMARY KEY,
perc_ciutadans_no_votat NUMBER(5,2) CONSTRAINT ck_perc_ciutadans_no_votat CHECK 
(perc_ciutadans_no_votat >= 0 and perc_ciutadans_no_votat <= 100)
);

--E7 Votaci que histricament ha tingut ms participaci
CREATE TABLE E7 (
id_E7 INTEGER CONSTRAINT PK_E7_id PRIMARY KEY,
id_votacio_participacio INTEGER CONSTRAINT NN_E7_votacio_participacio NOT NULL,
perc_participacio_e7 NUMBER(5,2) CONSTRAINT NN_E7_participacio NOT NULL
);

--E8 Votaci que histricament ha tingut ms diferncia percentual entre l'opci ms votada i la menys 
CREATE TABLE E8 (
id_E8 INTEGER CONSTRAINT PK_E8_id_E8 PRIMARY KEY,
id_votacio_mes_dif INTEGER CONSTRAINT NN_E8_id_votacio_mes_dif NOT NULL,
diferencia_e8 NUMBER(5,2) CONSTRAINT NN_E8_diferencia NOT NULL
);

--E9 Votaci que histricament ha tingut menys diferncia percentual entre l'opci ms votada i la menys 
CREATE TABLE E9 (
id_E9 INTEGER CONSTRAINT PK_E9_id_E9 PRIMARY KEY,
id_votacio_menys_dif INTEGER CONSTRAINT NN_E9_id_votacio_menys NOT NULL,
diferencia_e9 NUMBER(5,2) CONSTRAINT NN_E9_diferencia NOT NULL
);

--E10 Nmero mxim de votacions en les que ha participat una mateixa persona i el pas d'aquesta persona
CREATE TABLE E10 (
id_E10 INTEGER CONSTRAINT PK_E10_id_E10 PRIMARY KEY,
num_maxim_votacions NUMBER(10) CONSTRAINT NN_E10_num_maxim_votacions NOT NULL,
codi_pais_e10 CHAR(2) CONSTRAINT NN_E10_codi_pais NOT NULL,
codi_ciutada INTEGER CONSTRAINT NN_E10_codi_ciutada NOT NULL
);

--ALTERS QUE REFERENCIEN A CLAUS FORANES

-- Votaci
ALTER TABLE Votacio
ADD (
CONSTRAINT FK1_id_opcio_guanyadora FOREIGN KEY (id_opcio_guanyadora) REFERENCES Opcio (id_opcio),
CONSTRAINT UN_votacio_titol UNIQUE (titol)
);

-- Opci
ALTER TABLE Opcio
ADD CONSTRAINT UN_opcio_descripcio UNIQUE (descripcio);

-- Pas
ALTER TABLE Pais
ADD CONSTRAINT UN_pais_nom UNIQUE (nom_pais);

-- Regi
ALTER TABLE Regio
ADD CONSTRAINT UN_regio_nom UNIQUE (nom_regio);

-- CensGlobal
ALTER TABLE CensGlobal
ADD (
CONSTRAINT FK1_id_cens_global FOREIGN KEY (id_cens) REFERENCES Cens (id_cens),
CONSTRAINT UN_censglobal_edat UNIQUE (edat_minima)
);

-- CensLocal
ALTER TABLE CensLocal
ADD (
CONSTRAINT FK1_id_cens_local FOREIGN KEY (id_cens) REFERENCES Cens (id_cens),
CONSTRAINT FK2_id_localitat FOREIGN KEY (id_localitat) REFERENCES Localitat (id_localitat),
CONSTRAINT UN_censlocal_localitat UNIQUE (id_localitat)
);

-- CensRegional
ALTER TABLE CensRegional
ADD (
CONSTRAINT FK1_id_cens_regional FOREIGN KEY (id_cens) REFERENCES Cens (id_cens),
CONSTRAINT FK2_id_regio FOREIGN KEY (id_regio) REFERENCES Regio (id_regio),
CONSTRAINT UN_censregional_regio UNIQUE (id_regio)
);

-- CensEstatal
ALTER TABLE CensEstatal
ADD (
CONSTRAINT FK1_id_cens_estatal FOREIGN KEY (id_cens) REFERENCES Cens (id_cens),
CONSTRAINT FK2_codi_pais_estatal FOREIGN KEY (codi_pais) REFERENCES Pais (codi_pais),
CONSTRAINT UN_censestatal_pais UNIQUE (codi_pais)
);

-- Localitat
ALTER TABLE Localitat
ADD CONSTRAINT FK1_id_regio_localitat FOREIGN KEY (id_regio) REFERENCES Regio (id_regio);

-- Regi
ALTER TABLE Regio
ADD CONSTRAINT FK1_id_regio_pais FOREIGN KEY (codi_pais) REFERENCES Pais (codi_pais);

-- Ciutada
ALTER TABLE Ciutada
ADD CONSTRAINT FK1_id_localitat FOREIGN KEY (id_localitat) REFERENCES Localitat (id_localitat);

-- Assignacio
ALTER TABLE Assignacio
ADD (
CONSTRAINT FK1_id_votacio_assig FOREIGN KEY (id_votacio) REFERENCES Votacio (id_votacio),
CONSTRAINT FK2_codi_ciutada_assig FOREIGN KEY (codi_ciutada) REFERENCES Ciutada (codi_ciutada),
CONSTRAINT FK3_id_cens_assig FOREIGN KEY (id_cens) REFERENCES Cens (id_cens)
);

-- Vota
ALTER TABLE Vota
ADD (
CONSTRAINT FK1_id_votacio_vota FOREIGN KEY (id_votacio) REFERENCES Votacio (id_votacio),
CONSTRAINT FK3_codi_ciutada_vota FOREIGN KEY (codi_ciutada) REFERENCES Ciutada (codi_ciutada),
CONSTRAINT FK2_id_opcio_vota FOREIGN KEY (id_opcio) REFERENCES Opcio (id_opcio)
);

-- Opcio_Votacio
ALTER TABLE Opcio_Votacio
ADD (
CONSTRAINT FK1_id_opcio_ov FOREIGN KEY (id_opcio) REFERENCES Opcio (id_opcio),
CONSTRAINT FK2_id_votacio_ov FOREIGN KEY (id_votacio) REFERENCES Votacio (id_votacio)
);

-- E1
ALTER TABLE E1
ADD CONSTRAINT FK1_E1_codi_pais FOREIGN KEY (codi_pais) REFERENCES Pais (codi_pais);

-- E2
ALTER TABLE E2
ADD CONSTRAINT FK1_E2_id_cens FOREIGN KEY (id_cens) REFERENCES Cens (id_cens);

-- E3
ALTER TABLE E3
ADD CONSTRAINT FK1_E3_id_cens FOREIGN KEY (id_cens) REFERENCES Cens (id_cens);

-- E4
ALTER TABLE E4
ADD CONSTRAINT FK1_E4_id_votacio FOREIGN KEY (id_votacio) REFERENCES Votacio (id_votacio);

-- E5
ALTER TABLE E5
ADD CONSTRAINT FK1_E5_id_localitat FOREIGN KEY (id_localitat) REFERENCES Localitat (id_localitat);

-- E7
ALTER TABLE E7
ADD CONSTRAINT FK1_E7_id_votacio_participacio FOREIGN KEY (id_votacio_participacio) REFERENCES Votacio (id_votacio);

-- E8
ALTER TABLE E8
ADD CONSTRAINT FK1_E8_id_votacio_mes_dif FOREIGN KEY (id_votacio_mes_dif) REFERENCES Votacio (id_votacio);

-- E9
ALTER TABLE E9
ADD CONSTRAINT FK1_E9_id_votacio_menys_dif FOREIGN KEY (id_votacio_menys_dif) REFERENCES Votacio (id_votacio);

-- E10
ALTER TABLE E10
ADD (
CONSTRAINT FK1_E10_codi_pais FOREIGN KEY (codi_pais_e10) REFERENCES Pais (codi_pais),
CONSTRAINT FK2_E10_codi_ciutada FOREIGN KEY (codi_ciutada) REFERENCES Ciutada (codi_ciutada)
);
