--------------------------------------------------------------------------
-- Inserci del registre a la taula Logs -  
--			obtenim els registres dels procediments executats, els 
--			parmetres que ha rebut, i els que ha retornat
--
-- Parmetres d'entrada: nom_procediment, param_entrada, param_sortida, resultat
-- Parmetres de sortida:
-- Procediments que executa: 
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_log_insert (
	Pnom_procediment IN Logs.nom_procediment%TYPE, 
	Pparam_entrada IN Logs.param_entrada%TYPE,
	Pparam_sortida IN Logs.param_sortida%TYPE,
	Presultat IN Logs.resultat%TYPE
	)
IS
    campdatahora TIMESTAMP;
BEGIN
	campdatahora := SYSDATE;
    INSERT INTO Logs (nom_procediment, param_entrada, param_sortida, resultat, usuari_BD, datahora)
            VALUES (Pnom_procediment, Pparam_entrada, Pparam_sortida, Presultat, USER, campdatahora); 
END p_log_insert;
/


--------------------------------------------------------------------------
-- Canvi d'estat d'una votaci
--			es fa una actualitzaci del camp estat de la taula Votacio
--
-- Parmetres d'entrada: id_votacio, estat
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_canviestat_update (
	Pid_votacio IN votacio.id_votacio%TYPE, 
	Pestat IN votacio.estat%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_canviestat_update';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(100);
	noCodi EXCEPTION;
	noEstat EXCEPTION;
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pestat || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noCodi;
        END IF;
		IF (Pestat = 'E' OR Pestat = 'P' OR Pestat = 'O' OR Pestat = 'F') THEN
			NULL;
		ELSE
            RAISE noEstat;
        END IF;		
		-- Comprovaci que existeixi la votacio
			BEGIN 
				SELECT titol INTO nom FROM votacio
				WHERE Pid_votacio = id_votacio;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobat;
			WHEN OTHERS THEN
				RAISE altraExcepcio;
			END; 
	
        -- Update
        UPDATE votacio set 
			estat = Pestat
        WHERE id_votacio = Pid_votacio;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de votaci> incorrecte';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEstat THEN
				resultat := 'ERROR: <Estat> incorrecte';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_canviestat_update;
/

--------------------------------------------------------------------------
-- Finalitzacio d'una votaci - 
--			fa el recompte de la votaci
--			actualitza camps de vries taules
--			executa els procediments d'estadstica
--
-- Parmetres d'entrada: id_votacio
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert, p_opcio_votacio_update,
--		p_e1_estadistica, p_e2_estadistica, p_e3_estadistica, p_e4_estadistica
--		p_e5_estadistica, p_e6_estadistica, p_e7_estadistica, p_e8_estadistica
--		p_e9_estadistica, p_e10_estadistica
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_finalitzacio_votacio (
	Pid_votacio IN votacio.id_votacio%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_finalitzacio_votacio';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	edat INTEGER;
	votant INTEGER;	
	codireg INTEGER;		
	codi INTEGER;		
	estatvot CHAR(1);		
	num_assignats INTEGER;		
	num_votants INTEGER;			
	noVot EXCEPTION;
	noEstat EXCEPTION;	
	noTrobaVot EXCEPTION;	
	noTrobaCiu EXCEPTION;	
	noTrobaVotants EXCEPTION;
	noTrobaOpcions EXCEPTION;	
	noSeguir EXCEPTION;	
	altraExcepcio EXCEPTION;
	participacio NUMBER(5,2);	
	percentatge NUMBER(5,2);	
	perc_guanyador NUMBER(5,2);	
	perc_perdedor NUMBER(5,2);		
	diferencia NUMBER(5,2);			
	opcio INTEGER;	
	totalvots INTEGER;	
	num_opcions INTEGER;	
	guanyador INTEGER;	
	primer INTEGER;			
	anyconcret INTEGER;
	data_fi DATE;
	err_num NUMBER;
	err_msg Varchar2(255);
	RSP Varchar2(100);
	-- Definim el cursor
	CURSOR c_finalitzacio IS
	-- Calcula el total de vots que ha tret cada opcio votada 
		Select distinct id_opcio, count(*) as total
		from vota
		where id_votacio = Pid_votacio
		group by id_opcio
		order by total desc;
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
		RSP := 'OK';
        entrada := '<' || Pid_votacio || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
		-- Comprovaci de que la votacio existeixi
        BEGIN 
            SELECT id_votacio, estat, data_final INTO codi, estatvot, data_fi 
			FROM Votacio
            WHERE Pid_votacio = id_votacio;
		IF (estatvot = 'F') THEN RAISE noEstat;
        END IF;		
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Calcula el nmero de persones amb capacitat de vot
        BEGIN 
			Select count(codi_ciutada) INTO num_assignats
			from assignacio
			where id_votacio = pid_votacio;		
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaCiu;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Calcula el nmero total ciutadans que han votat
		BEGIN
			Select count(codi_ciutada) INTO num_votants
			from vota
			where id_votacio = Pid_votacio;			
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVotants;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Calculem el percentatge de participacio
		participacio := (num_votants * 100) / num_assignats; 

		-- Calcula el nmero total d'opcions que t la votaci
		BEGIN
			Select count(*) INTO num_opcions
			from opcio_votacio
			where id_votacio = Pid_votacio;			
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaOpcions;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;

		-- Bucle principal del cursor
		BEGIN
			-- Tractament del cursor
			OPEN c_finalitzacio;
			primer := 1;
			perc_perdedor := 0;
			LOOP
				FETCH c_finalitzacio INTO opcio, totalvots;
				EXIT WHEN c_finalitzacio%NOTFOUND;
				-- Calculem el percentatge sobre el total
				percentatge := (totalvots * 100) / num_votants;
				perc_perdedor := percentatge;
				IF primer = 1 THEN 
					guanyador := opcio;
					perc_guanyador := percentatge;
					perc_perdedor := percentatge;
					primer := primer + 1;
				END IF;
				p_opcio_votacio_update(opcio, Pid_votacio, totalvots, percentatge, RSP);
				IF RSP <> 'OK' THEN 
					CLOSE c_finalitzacio;
					RAISE noSeguir;
				END IF;
			END LOOP;
			IF (c_finalitzacio%ROWCOUNT > num_opcions) THEN 
				perc_perdedor := 0;
			END IF;
			CLOSE c_finalitzacio;
		END;

		-- Actualitzem els camps estadstics de la votaci
        UPDATE Votacio set 
			estat = 'F',
			perc_participacio = participacio,
			num_pers_capacitat_vot = num_assignats,
			vots_totals = num_votants,
			perc_vots_opcio_guanyadora = perc_guanyador,
			id_opcio_guanyadora = guanyador
        WHERE id_votacio = Pid_votacio;

		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
		
		-- Actualitzem les estadstiques	
		diferencia := perc_guanyador - perc_perdedor;
		anyconcret := TO_NUMBER(TO_CHAR(data_fi,'YYYY'));	
		p_e1_estadistica(Pid_votacio, anyconcret, RSP);
		p_e2_estadistica(anyconcret, RSP);
		p_e3_estadistica(Pid_votacio, anyconcret, RSP);
		p_e4_estadistica(Pid_votacio, anyconcret, participacio, RSP);
		p_e5_estadistica(anyconcret, RSP);
		p_e6_estadistica(anyconcret, RSP);
		p_e7_estadistica(Pid_votacio, participacio, RSP);
		p_e8_estadistica(Pid_votacio, diferencia, RSP);
		p_e9_estadistica(Pid_votacio, diferencia, RSP);
		p_e10_estadistica(RSP);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEstat THEN
				resultat := 'ERROR: La votaci ja ha est finalitzada';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaCiu THEN
				resultat := 'ERROR: No hi ha ciutadans assignats';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVotants THEN
				resultat := 'ERROR: No hi ha cap ciutada que hagi votat';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaOpcions THEN
				resultat := 'ERROR: No hi ha opcions a opcio_votaci';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noSeguir THEN
				resultat := 'ERROR: Taula Opcio_Votacio mal actualitzada, mirar log';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_finalitzacio_votacio;
/


---------------------------------------------------------------------
-- Utilitat noms per proves - 	Canvi data dels vots d'una votaci a 
--								la taula Vota
--
-- Parmetres d'entrada: id_votacio, data_nova
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
---------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_util_data_vota (
	Pid_votacio IN votacio.id_votacio%TYPE, 
	Pdata_nova IN TIMESTAMP,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_util_data_vota';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(100);
	estatvot CHAR(1);
	datanova TIMESTAMP;
	noCodi EXCEPTION;
	noFinalitzada EXCEPTION;
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	noReg EXCEPTION;	
	err_num NUMBER;
	err_msg Varchar2(255);	
	CURSOR c_vota IS
		SELECT data_hora_vot 
		FROM Vota
		WHERE id_votacio = Pid_votacio
		FOR UPDATE of data_hora_vot;
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pdata_nova || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noCodi;
        END IF;
		-- Comprovaci que existeixi la votacio
			BEGIN 
				SELECT estat INTO estatvot FROM votacio
				WHERE Pid_votacio = id_votacio;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobat;
			WHEN OTHERS THEN
				RAISE altraExcepcio;
			END; 
--		IF estatvot <> 'F' THEN 
--				RAISE noFinalitzada;
--		END IF;
        -- Tractament del cursor
		BEGIN
			-- Tractament del cursor
			OPEN c_vota;
			LOOP
				FETCH c_vota INTO datanova;
				EXIT WHEN c_vota%NOTFOUND;
				UPDATE Vota
					SET data_hora_vot = Pdata_nova
				WHERE CURRENT OF c_vota;
			END LOOP;
			IF (c_vota%ROWCOUNT < 1) THEN 
				RAISE noReg;
			END IF;
			CLOSE c_vota;
		END;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de votaci> incorrecte';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noFinalitzada THEN
				resultat := 'ERROR: La votaci no est finalitzada';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noReg THEN
				resultat := 'ERROR: No ha trobat registres';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_util_data_vota;
/
---------------------------------------------------------------------
-- Utilitat noms per proves - 	Canvi data d'assignaci d'un cens a 
--								la taula Assignaci
--
-- Parmetres d'entrada: id_votacio, data_nova
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
---------------------------------------------------------------------

CREATE OR REPLACE PROCEDURE p_util_data_assig (
	Pid_votacio IN votacio.id_votacio%TYPE, 
	Pdata_nova IN TIMESTAMP,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_util_data_assig';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(100);
	estatvot CHAR(1);
	datanova TIMESTAMP;
	noCodi EXCEPTION;
	noFinalitzada EXCEPTION;
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	noReg EXCEPTION;	
	err_num NUMBER;
	err_msg Varchar2(255);	
	CURSOR c_assig IS
		SELECT data_assignacio 
		FROM Assignacio
		WHERE id_votacio = Pid_votacio
		FOR UPDATE of data_assignacio;
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pdata_nova || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noCodi;
        END IF;
		-- Comprovaci que existeixi la votacio
			BEGIN 
				SELECT estat INTO estatvot FROM votacio
				WHERE Pid_votacio = id_votacio;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobat;
			WHEN OTHERS THEN
				RAISE altraExcepcio;
			END; 
--		IF estatvot <> 'F' THEN 
--				RAISE noFinalitzada;
--		END IF;
        -- Tractament del cursor
		BEGIN
			-- Tractament del cursor
			OPEN c_assig;
			LOOP
				FETCH c_assig INTO datanova;
				EXIT WHEN c_assig%NOTFOUND;
				UPDATE Assignacio
					SET data_assignacio = Pdata_nova
				WHERE CURRENT OF c_assig;
			END LOOP;
			IF (c_assig%ROWCOUNT < 1) THEN 
				RAISE noReg;
			END IF;
			CLOSE c_assig;
		END;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de votaci> incorrecte';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noFinalitzada THEN
				resultat := 'ERROR: La votaci no est finalitzada';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noReg THEN
				resultat := 'ERROR: No ha trobat registres';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_util_data_assig;
/
