-----------Procedures d'ABM (Altes-Baixes-Modificaci)-----------------------

------------------------------------------------------
-- Pas - insert
-- Parmetres d'entrada: codi_pais, nom_pais
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_pais_insert (
	Pcodi_pais IN pais.codi_pais%TYPE, 
	Pnom_pais IN pais.nom_pais%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_pais_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noCodi EXCEPTION;
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	noNom EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pcodi_pais || '> <' || Pnom_pais || '>';
        -- Comprovaci parmetres entrada
        IF (Pcodi_pais IS NULL OR Pcodi_pais = '') THEN
            RAISE noCodi;
        END IF;
        IF (Pnom_pais IS NULL OR Pnom_pais = '') THEN
            RAISE noNom;
        END IF;
        
        -- Inserci a la taula
        INSERT INTO pais (codi_pais, nom_pais)
                   VALUES (Pcodi_pais, Pnom_pais);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de pas> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noNom THEN
				resultat := 'ERROR: <Nom pas> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: El pas no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Pas ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);			
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_pais_insert;
/
------------------------------------------------------
-- Pas - update
-- Parmetres d'entrada: codi_pais, nom_pais
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_pais_update (
	Pcodi_pais IN pais.codi_pais%TYPE, 
	Pnom_pais IN pais.nom_pais%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_pais_update';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(100);
	noCodi EXCEPTION;
	noNom EXCEPTION;
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);

BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pcodi_pais || '> <' || Pnom_pais || '>';
        -- Comprovaci parmetres entrada
        IF (Pcodi_pais IS NULL OR Pcodi_pais = '') THEN
            RAISE noCodi;
        END IF;
        IF (Pnom_pais IS NULL OR Pnom_pais = '') THEN
            RAISE noNom;
        END IF;
		-- Comprovaci que existeixi el pas
			BEGIN 
				SELECT nom_pais INTO nom FROM pais
				WHERE Pcodi_pais = codi_pais;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobat;
			WHEN OTHERS THEN
				RAISE altraExcepcio;
			END; 
	
        -- Update
        UPDATE pais set 
			nom_pais = Pnom_pais
        WHERE codi_pais = Pcodi_pais;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de pas> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noNom THEN
				resultat := 'ERROR: <Nom pas> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: El pas no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_pais_update;
/
------------------------------------------------------
-- Pas - delete
-- Parmetres d'entrada: codi_pais
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_pais_delete (
	Pcodi_pais IN pais.codi_pais%TYPE,
	resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_pais_delete';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(50);
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	noCodi EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pcodi_pais || '>';
		        
        -- Comprobaci parmetres entrada
        IF (Pcodi_pais IS NULL OR Pcodi_pais = '') THEN
		    RAISE noCodi;
        END IF;
        BEGIN 
            SELECT nom_pais INTO nom FROM Pais
            WHERE Pcodi_pais = codi_pais;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
	
        -- Eliminaci
        DELETE FROM pais
         WHERE codi_pais = Pcodi_pais;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
			resultat := 'ERROR: <Codi de pas> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobat THEN
				resultat := 'ERROR: El pas no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_pais_delete;
/
--------------------------------------------------------------
-- Ciutad - insert
-- Parmetres d'entrada: codi_ciutada, id_localitat, data_naixament
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
--------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_ciutada_insert (
	Pcodi_ciutada IN ciutada.codi_ciutada%TYPE,
	Pid_localitat IN ciutada.id_localitat%TYPE,
	Pdata_naixament IN ciutada.data_naixament%TYPE, 
	resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_ciutada_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	codiLoc INTEGER;
	noCodi EXCEPTION;
	noLocalitat EXCEPTION;
	noTrobat EXCEPTION;
	noData EXCEPTION;
	altraExcepcio EXCEPTION;
	campdataalta DATE;
	err_num NUMBER;
	err_msg Varchar2(255);
BEGIN
        resultat := 'OK';
		campdataalta := SYSDATE;
        entrada := '<'   || Pcodi_ciutada  || '> <' || Pid_localitat || 
		   '> <' || Pdata_naixament || '>';
        -- Comprovaci parmetres entrada
        IF (Pcodi_ciutada IS NULL OR Pcodi_ciutada = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pid_localitat IS NULL OR Pid_localitat = 0) THEN
            RAISE noLocalitat;
        END IF;
        IF (Pdata_naixament IS NULL) THEN
            RAISE noData;
        END IF;
		-- Comprovem que la localitat existeixi
        BEGIN 
            SELECT id_localitat INTO codiLoc FROM Localitat
            WHERE Pid_localitat = id_localitat;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
			WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
        -- Inserci
        INSERT INTO Ciutada (codi_ciutada, id_localitat, data_naixament, data_alta, data_baixa)
                   VALUES (Pcodi_ciutada, Pid_localitat, Pdata_naixament, campdataalta, NULL);

		-- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
			resultat := 'ERROR: <Codi de ciutad> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobat THEN
				resultat := 'ERROR: La localitat no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLocalitat THEN
				resultat := 'ERROR: <Codi localitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noData THEN
				resultat := 'ERROR: <Data naixament> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Ciutad ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_ciutada_insert;
/

--------------------------------------------------------------
-- Ciutad - update
-- Parmetres d'entrada: codi_ciutada, id_localitat, data_naixament
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
--------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_ciutada_update (
	Pcodi_ciutada IN ciutada.codi_ciutada%TYPE,
	Pid_localitat IN ciutada.id_localitat%TYPE,
	Pdata_naixament IN ciutada.data_naixament%TYPE, 
	resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_ciutada_update';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	codiLoc INTEGER;
	codiCiu INTEGER;	
	noCodi EXCEPTION;
	noLocalitat EXCEPTION;
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
    noData EXCEPTION;	
	err_num NUMBER;
	err_msg Varchar2(255);
BEGIN
        resultat := 'OK'; 
        entrada := '<'   || Pcodi_ciutada  || '> <' || Pid_localitat || 
		   '> <' || Pdata_naixament || '>';
        -- Comprovaci parmetres entrada
        IF (Pcodi_ciutada IS NULL OR Pcodi_ciutada = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pid_localitat IS NULL OR Pid_localitat = 0) THEN
            RAISE noLocalitat;
        END IF;
        IF (Pdata_naixament IS NULL) THEN
            RAISE noData;
        END IF;

		-- Comprovem que la localitat existeixi
        BEGIN 
            SELECT id_localitat INTO codiLoc FROM Localitat
            WHERE Pid_localitat = id_localitat;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noLocalitat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 

		-- Comprovem que el ciutad existeixi
        BEGIN 
            SELECT Pcodi_ciutada INTO codiCiu FROM Ciutada
            WHERE Pcodi_ciutada = codi_ciutada;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 

        -- Update
        UPDATE Ciutada SET 
			id_localitat = Pid_localitat,
			data_naixament = Pdata_naixament
        WHERE codi_ciutada = Pcodi_ciutada;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
		EXCEPTION
			WHEN noCodi THEN
			resultat := 'ERROR: <Codi de ciutad> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobat THEN
				resultat := 'ERROR: El ciutad no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLocalitat THEN
				resultat := 'ERROR: <Codi localitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noData THEN
				resultat := 'ERROR: <Data naixament> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_ciutada_update;
/
--------------------------------------------------------------
-- Ciutad - delete
-- Parmetres d'entrada: codi_ciutada
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
--------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_ciutada_delete (
    Pcodi_ciutada IN ciutada.codi_ciutada%TYPE,
	resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_ciutada_delete';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	codiLoc Integer;
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	noCodi EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pcodi_ciutada || '>';
		        
        -- Comprobaci parmetres entrada
        IF (Pcodi_ciutada IS NULL OR Pcodi_ciutada = '') THEN
		    RAISE noCodi;
        END IF;
        BEGIN 
            SELECT codi_ciutada INTO codiLoc FROM Ciutada
            WHERE Pcodi_ciutada = codi_ciutada;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
	
        -- Eliminaci, que grava la data del sistema al camp data_baixa
        UPDATE Ciutada SET 
			data_baixa = SYSDATE
        WHERE codi_ciutada = Pcodi_ciutada;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
			resultat := 'ERROR: <Codi de ciutad> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobat THEN
				resultat := 'ERROR: El ciutad no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_ciutada_delete;
/

------------------------------------------------------
-- Regi - insert
-- Parmetres d'entrada: codi_pais, nom_regio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_regio_insert (
	Pcodi_pais IN regio.codi_pais%TYPE, 
	Pnom_regio IN regio.nom_regio%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_regio_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(50);
	noCodi EXCEPTION;
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	noNom EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pcodi_pais || '> <' || Pnom_regio || '>';
        -- Comprovaci parmetres entrada
        IF (Pcodi_pais IS NULL OR Pcodi_pais = '') THEN
            RAISE noCodi;
        END IF;
        IF (Pnom_regio IS NULL OR Pnom_regio = '') THEN
            RAISE noNom;
        END IF;
		-- Obtenim el nom del pas
        BEGIN 
            SELECT nom_pais INTO nom FROM Pais
            WHERE Pcodi_pais = codi_pais;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
		-- Comprovem que no existeixi una regi amb el mateix
		-- nom i pas
--        BEGIN 
--            SELECT nom_regio INTO nom FROM Regio
--            WHERE Pcodi_pais = codi_pais
--			AND Pnom_regio = nom_regio;
--        EXCEPTION
--        WHEN NO_DATA_FOUND THEN
--            RAISE noTrobat;
--        WHEN OTHERS THEN
--            RAISE altraExcepcio;
--        END; 
        -- Inserci a la taula
        INSERT INTO Regio (codi_pais, nom_regio)
                   VALUES (Pcodi_pais, Pnom_regio);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de pas> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noNom THEN
				resultat := 'ERROR: <Nom regi> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: El pas no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_regio_insert;
/
------------------------------------------------------
-- Regi - update
-- Parmetres d'entrada: id_regio, codi_pais, nom_regio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_regio_update (
	Pid_regio IN regio.id_regio%TYPE, 
	Pcodi_pais IN regio.codi_pais%TYPE, 	
	Pnom_regio IN regio.nom_regio%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_regio_update';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(100);
	noCodi EXCEPTION;
	noNom EXCEPTION;
	noPais EXCEPTION;	
	noTrobat EXCEPTION;
	noTrobaPais EXCEPTION;
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);

BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_regio || '> <' || Pcodi_pais || '> <' || Pnom_regio || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_regio IS NULL OR Pid_regio = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pcodi_pais IS NULL OR Pcodi_pais = '') THEN
            RAISE noPais;
        END IF;
        IF (Pnom_regio IS NULL OR Pnom_regio = '') THEN
            RAISE noNom;
        END IF;
		-- Comprovaci que existeixi la regi
			BEGIN 
				SELECT nom_regio INTO nom FROM regio
				WHERE Pid_regio = id_regio;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobat;
			WHEN OTHERS THEN
				RAISE altraExcepcio;
			END; 
		-- Comprovaci que existeixi el pas
			BEGIN 
				SELECT nom_pais INTO nom FROM Pais
				WHERE Pcodi_pais = codi_pais;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobaPais;
			WHEN OTHERS THEN
				RAISE altraExcepcio;
			END; 
	
        -- Update
        UPDATE regio set 
			codi_pais = Pcodi_pais,	
			nom_regio = Pnom_regio
        WHERE id_regio = Pid_regio;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de regio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noNom THEN
				resultat := 'ERROR: <Nom regi> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noPais THEN
				resultat := 'ERROR: <Codi pas> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: La regi no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaPais THEN
				resultat := 'ERROR: El pas no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_regio_update;
/
------------------------------------------------------
-- Regi - delete
-- Parmetres d'entrada: id_regio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_regio_delete (
	Pid_regio IN regio.id_regio%TYPE,
	resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_regio_delete';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(50);
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	noCodi EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_regio || '>';
		        
        -- Comprobaci parmetres entrada
        IF (Pid_regio IS NULL OR Pid_regio = 0) THEN
		    RAISE noCodi;
        END IF;
        BEGIN 
            SELECT nom_regio INTO nom FROM Regio
            WHERE Pid_regio = id_regio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
	
        -- Eliminaci
        DELETE FROM regio
         WHERE id_regio = Pid_regio;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
			resultat := 'ERROR: <Codi de regi> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobat THEN
				resultat := 'ERROR: La regi no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_regio_delete;
/

------------------------------------------------------
-- Localitat - insert
-- Parmetres d'entrada: id_regio, nom_localitat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_localitat_insert (
	Pid_regio IN localitat.id_regio%TYPE, 
	Pnom_localitat IN localitat.nom_localitat%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_localitat_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(50);
	noCodi EXCEPTION;
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	noNom EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_regio || '> <' || Pnom_localitat || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_regio IS NULL OR Pid_regio = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pnom_localitat IS NULL OR Pnom_localitat = '') THEN
            RAISE noNom;
        END IF;
		-- Comprovaci que existeixi la regi
        BEGIN 
            SELECT nom_regio INTO nom FROM Regio
            WHERE Pid_regio = id_regio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
        -- Inserci a la taula
        INSERT INTO Localitat (id_regio, nom_localitat)
                   VALUES (Pid_regio, Pnom_localitat);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de regi> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noNom THEN
				resultat := 'ERROR: <Nom de localitat> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: La regi no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_localitat_insert;
/
------------------------------------------------------
-- Localitat - update
-- Parmetres d'entrada: id_localitat, id_regio, nom_localitat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_localitat_update (
	Pid_localitat IN localitat.id_localitat%TYPE, 
	Pid_regio IN localitat.id_regio%TYPE, 	
	Pnom_localitat IN localitat.nom_localitat%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_localitat_update';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(100);
	noLoc EXCEPTION;
	noReg EXCEPTION;
	noNom EXCEPTION;
	noPais EXCEPTION;	
	noTrobat EXCEPTION;
	noTrobaRegio EXCEPTION;
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);

BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_localitat || '> <' || Pid_regio || '> <' || Pnom_localitat || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_localitat IS NULL OR Pid_localitat = 0) THEN
            RAISE noLoc;
        END IF;
        IF (Pid_regio IS NULL OR Pid_regio = 0) THEN
            RAISE noReg;
        END IF;
        IF (Pnom_localitat IS NULL OR Pnom_localitat = '') THEN
            RAISE noNom;
        END IF;
		-- Comprovaci que existeixi la localitat
			BEGIN 
				SELECT nom_localitat INTO nom FROM Localitat
				WHERE Pid_localitat = id_localitat;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobat;
			WHEN OTHERS THEN
				RAISE altraExcepcio;
			END; 
		-- Comprovaci que existeixi la regi
			BEGIN 
				SELECT nom_regio INTO nom FROM Regio
				WHERE Pid_regio = id_regio;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobaRegio;
			WHEN OTHERS THEN
				RAISE altraExcepcio;
			END; 
	
        -- Update
        UPDATE localitat set 
			id_regio = Pid_regio,	
			nom_localitat = Pnom_localitat
        WHERE id_localitat = Pid_localitat;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noLoc THEN
				resultat := 'ERROR: <Codi de localitat> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noNom THEN
				resultat := 'ERROR: <Nom localitat> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noReg THEN
				resultat := 'ERROR: <Codi de regi> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: La localitat no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaRegio THEN
				resultat := 'ERROR: La regi no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_localitat_update;
/
------------------------------------------------------
-- Localitat - delete
-- Parmetres d'entrada: id_localitat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_localitat_delete (
	Pid_localitat IN localitat.id_localitat%TYPE,
	resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_localitat_delete';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(50);
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	noCodi EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_localitat || '>';
		        
        -- Comprobaci parmetres entrada
        IF (Pid_localitat IS NULL OR Pid_localitat = 0) THEN
		    RAISE noCodi;
        END IF;
		-- Comprovem que existeixi la Localitat
        BEGIN 
            SELECT nom_localitat INTO nom FROM localitat
            WHERE Pid_localitat = id_localitat;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
	
        -- Eliminaci
        DELETE FROM localitat
         WHERE id_localitat = Pid_localitat;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
			resultat := 'ERROR: <Codi de localitat> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobat THEN
				resultat := 'ERROR: La localitat no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_localitat_delete;
/

------------------------------------------------------
-- Cens - insert
-- Parmetres d'entrada: tipus_cens
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_cens_insert (
	Ptipus_cens IN cens.tipus_cens%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_cens_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noTipus EXCEPTION;
	noTipus2 EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Ptipus_cens || '> ';
        -- Comprovaci parmetres entrada
        IF (Ptipus_cens IS NULL OR Ptipus_cens = ' ') THEN
            RAISE noTipus;
        END IF;
        
		IF (Ptipus_cens = 'G' OR Ptipus_cens = 'E' OR Ptipus_cens = 'L' OR Ptipus_cens = 'R') THEN
			NULL;
		ELSE
            RAISE noTipus2;
        END IF;
        
        -- Inserci a la taula
        INSERT INTO Cens (tipus_cens)
                   VALUES (Ptipus_cens);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noTipus THEN
				resultat := 'ERROR: <Tipus de cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTipus2 THEN
				resultat := 'ERROR: El tipus de cens no existeix';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_cens_insert;
/
------------------------------------------------------
-- Cens - update
-- Parmetres d'entrada: id_cens, tipus_cens
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_cens_update (
	Pid_cens IN cens.id_cens%TYPE, 
	Ptipus_cens IN cens.tipus_cens%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_cens_update';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(100);
	noCodi EXCEPTION;
	noTipus EXCEPTION;
	noTipus2 EXCEPTION;	
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	tipus CHAR(1);
	err_num NUMBER;
	err_msg Varchar2(255);

BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_cens || '> <' || Ptipus_cens || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Ptipus_cens IS NULL OR Ptipus_cens = '') THEN
            RAISE noTipus;
        END IF;
		IF (Ptipus_cens = 'G' OR Ptipus_cens = 'E' OR Ptipus_cens = 'L' OR Ptipus_cens = 'R') THEN
			NULL;
		ELSE
            RAISE noTipus2;
        END IF;
		-- Comprovaci que existeixi el cens
			BEGIN 
				SELECT tipus_cens INTO tipus FROM Cens
				WHERE Pid_cens = id_cens;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobat;
			WHEN OTHERS THEN
				RAISE altraExcepcio;
			END; 
        -- Update
        UPDATE cens set 
			tipus_cens = Ptipus_cens
        WHERE id_cens = Pid_cens;
        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);
        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTipus THEN
				resultat := 'ERROR: <Tipus de cens> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTipus2 THEN
				resultat := 'ERROR: Tipus de cens no correspon';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobat THEN
				resultat := 'ERROR: El cens no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_cens_update;
/
------------------------------------------------------
-- Cens - delete
-- Parmetres d'entrada: id_cens
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_cens_delete (
	Pid_cens IN cens.id_cens%TYPE,
	resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_cens_delete';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(50);
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	noCodi EXCEPTION;
	tipus CHAR(1);
	err_num NUMBER;
	err_msg Varchar2(255);
	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_cens || '>';
        -- Comprobaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
		    RAISE noCodi;
        END IF;
        BEGIN 
            SELECT tipus_cens INTO tipus FROM Cens
            WHERE Pid_cens = id_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
        -- Eliminaci
        DELETE FROM cens
        WHERE Pid_cens = id_cens;
        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);
        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
			resultat := 'ERROR: <Codi de cens> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobat THEN
				resultat := 'ERROR: El cens no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_cens_delete;
/

------------------------------------------------------
-- Cens Global - insert
-- Parmetres d'entrada: id_cens, edat_minima
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_censglobal_insert (
	Pid_cens IN CensGlobal.id_cens%TYPE, 
	Pedat_minima IN CensGlobal.edat_minima%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_censglobal_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noCodi EXCEPTION;
	noEdat EXCEPTION;
	noTrobat EXCEPTION;
	noTipus EXCEPTION;
	altraExcepcio EXCEPTION;
	noNom EXCEPTION;
    tipus CHAR(1);
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		tipus:= ' ';
		resultat := 'OK';
        entrada := '<' || Pid_cens || '> <' || Pedat_minima || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pedat_minima IS NULL OR Pedat_minima = 0) THEN
            RAISE noEdat;
        END IF;

        BEGIN 
            SELECT tipus_cens INTO tipus FROM Cens
            WHERE Pid_cens = id_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
		
		IF tipus <> 'G' THEN RAISE noTipus;        
		END IF;
		
        -- Inserci a la taula
        INSERT INTO CensGlobal (id_cens, edat_minima)
                   VALUES (Pid_cens, Pedat_minima);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTipus THEN
				resultat := 'ERROR: <Tipus de cens> no correspon';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: El cens no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_censglobal_insert;
/
------------------------------------------------------
-- Cens Estatal - insert
-- Parmetres d'entrada: id_cens, codi_pais
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_censestatal_insert (
	Pid_cens IN CensEstatal.id_cens%TYPE, 
	Pcodi_pais IN CensEstatal.codi_pais%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_censestatal_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(50);
	noCodi EXCEPTION;
	noPais EXCEPTION;
	noTrobat EXCEPTION;
	noTrobatPais EXCEPTION;	
	noTipus EXCEPTION;	
	altraExcepcio EXCEPTION;
	noNom EXCEPTION;
	tipus CHAR(1);
	err_num NUMBER;
	err_msg Varchar2(255);
BEGIN
		-- Inicialitzaci de variables
		tipus:= ' ';
		resultat := 'OK';
        entrada := '<' || Pid_cens || '> <' || Pcodi_pais || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pcodi_pais IS NULL OR Pcodi_pais = ' ') THEN
            RAISE noPais;
        END IF;

        BEGIN 
            SELECT nom_pais INTO nom FROM Pais
            WHERE Pcodi_pais = codi_pais;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobatPais;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
        BEGIN 
            SELECT tipus_cens INTO tipus FROM Cens
            WHERE Pid_cens = id_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 

		IF tipus <> 'E' THEN RAISE noTipus;        
		END IF;
		
        -- Inserci a la taula
        INSERT INTO CensEstatal (id_cens, codi_pais)
                   VALUES (Pid_cens, Pcodi_pais);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <Codi de pais> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTipus THEN
				resultat := 'ERROR: <Tipus de cens> no correspont';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: El cens no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatPais THEN
				resultat := 'ERROR: <Codi pas> no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_censestatal_insert;
/
------------------------------------------------------
-- Cens Regional - insert
-- Parmetres d'entrada: id_cens, id_regio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_censregional_insert (
	Pid_cens IN CensRegional.id_cens%TYPE, 
	Pid_regio IN CensRegional.id_regio%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_censregional_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(50);
	noCodi EXCEPTION;
	noRegio EXCEPTION;
	noTrobat EXCEPTION;
	noTrobatRegio EXCEPTION;	
	noTipus EXCEPTION;	
	altraExcepcio EXCEPTION;
	noNom EXCEPTION;
	tipus CHAR(1);
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		tipus:= ' ';
		resultat := 'OK';
        entrada := '<' || Pid_cens || '> <' || Pid_regio || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pid_regio IS NULL OR Pid_regio = 0) THEN
            RAISE noRegio;
        END IF;

        BEGIN 
            SELECT nom_regio INTO nom FROM Regio
            WHERE Pid_regio = id_regio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobatRegio;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
        BEGIN 
            SELECT tipus_cens INTO tipus FROM Cens
            WHERE Pid_cens = id_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 

		IF tipus <> 'R' THEN RAISE noTipus;
		END IF;
		
        -- Inserci a la taula
        INSERT INTO CensRegional (id_cens, id_regio)
                   VALUES (Pid_cens, Pid_regio);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de cens> incorrecte';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noRegio THEN
				resultat := 'ERROR: <Codi de regio> incorrecte';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTipus THEN
				resultat := 'ERROR: <Tipus de cens> no correspont';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: El cens no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatRegio THEN
				resultat := 'ERROR: <Codi regi> no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_censregional_insert;
/
------------------------------------------------------
-- Cens Local - insert
-- Parmetres d'entrada: id_cens, id_localitat
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_censlocal_insert (
	Pid_cens IN CensLocal.id_cens%TYPE, 
	Pid_localitat IN CensLocal.id_localitat%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_censlocal_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(80);
	noCodi EXCEPTION;
	noLocalitat EXCEPTION;
	noTrobat EXCEPTION;
	noTrobatLoc EXCEPTION;	
	noTipus EXCEPTION;	
	altraExcepcio EXCEPTION;
	tipus CHAR(1);
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		tipus:= ' ';
		resultat := 'OK';
        entrada := '<' || Pid_cens || '> <' || Pid_localitat || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pid_localitat IS NULL OR Pid_localitat = 0) THEN
            RAISE noLocalitat;
        END IF;

        BEGIN 
            SELECT nom_localitat INTO nom FROM Localitat
            WHERE Pid_localitat = id_localitat;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobatLoc;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
        BEGIN 
            SELECT tipus_cens INTO tipus FROM Cens
            WHERE Pid_cens = id_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 

		IF tipus <> 'L' THEN RAISE noTipus;        
		END IF;
		
        -- Inserci a la taula
        INSERT INTO CensLocal (id_cens, id_localitat)
                   VALUES (Pid_cens, Pid_localitat);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi de cens> incorrecte';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noLocalitat THEN
				resultat := 'ERROR: <Codi de localitat> incorrecte';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTipus THEN
				resultat := 'ERROR: <Tipus de cens> no correspont';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: El cens no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobatLoc THEN
				resultat := 'ERROR: <Codi localitat> no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_censlocal_insert;
/
--------------------------------------------------------------
-- Votaci - insert
-- Parmetres d'entrada: titol, descripcio, data_inicial, data_final
--			data_publicaio, president, estat, web
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
--------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_votacio_insert (
	Ptitol IN votacio.titol%TYPE,
	Pdescripcio IN votacio.descripcio%TYPE,
	Pdata_inicial IN votacio.data_inicial%TYPE, 
	Pdata_final IN votacio.data_final%TYPE, 	
	Pdata_publicacio IN votacio.data_publicacio%TYPE, 	
	Ppresident IN votacio.president%TYPE, 	
	Pestat IN votacio.estat%TYPE, 
	Pweb IN votacio.web%TYPE, 	
	resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_votacio_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	err_num NUMBER;
	err_msg Varchar2(255);
	noTitol EXCEPTION;
	noDescripcio EXCEPTION;
	noDataI EXCEPTION;
	noDataF EXCEPTION;
	noDataP EXCEPTION;
	noPresident EXCEPTION;	
	noEstat EXCEPTION;
BEGIN
        resultat := 'OK';
        entrada := '<'   || Ptitol  || '> <' || Pdescripcio || 
		   '> <' || Pdata_inicial || '> <' || Pdata_final || '> <' || Pdata_publicacio ||
		   '> <' || Ppresident || '> <' || Pestat || '> <' || Pweb ||'>';
        -- Comprovaci parmetres entrada
        IF (Ptitol IS NULL OR Ptitol = ' ') THEN
            RAISE noTitol;
        END IF;
        IF (Pdescripcio IS NULL OR Pdescripcio = ' ') THEN
            RAISE noDescripcio;
        END IF;
        IF (Pdata_inicial IS NULL) THEN
            RAISE noDataI;
        END IF;
        IF (Pdata_final IS NULL) THEN
            RAISE noDataF;
        END IF;
        IF (Pdata_publicacio IS NULL) THEN
            RAISE noDataP;
        END IF;
        IF (Ppresident IS NULL OR Ppresident = ' ') THEN
            RAISE noPresident;
        END IF;
		IF (Pestat = 'E' OR Pestat = 'P' OR Pestat = 'O' OR Pestat = 'F') THEN
			NULL;
		ELSE
            RAISE noEstat;
        END IF;
        -- Inserci
        INSERT INTO Votacio (titol, descripcio, data_inicial, data_final, data_publicacio, president, estat, 
			web, perc_participacio, num_pers_capacitat_vot, vots_totals, id_opcio_guanyadora, perc_vots_opcio_guanyadora)
            VALUES (Ptitol, Pdescripcio, Pdata_inicial, Pdata_final, Pdata_publicacio, 
					Ppresident, Pestat, Pweb, 0, NULL, NULL, NULL, 0);

		-- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noTitol THEN
			resultat := 'ERROR: <Titol> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDescripcio THEN
				resultat := 'ERROR: <Descripci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataI THEN
				resultat := 'ERROR: <Data inicial> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataF THEN
				resultat := 'ERROR: <Data final> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataP THEN
				resultat := 'ERROR: <Data publicaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPresident THEN
				resultat := 'ERROR: <President> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noEstat THEN
				resultat := 'ERROR: <Estat> incorrecte';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Votaci ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_votacio_insert;
/
--------------------------------------------------------------
-- Votaci - update
-- Parmetres d'entrada: id_votacio, titol, descripcio, data_inicial, data_final
--			data_publicaio, president, estat, web
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
--------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_votacio_update (
	Pid_votacio IN votacio.id_votacio%TYPE,
	Ptitol IN votacio.titol%TYPE,
	Pdescripcio IN votacio.descripcio%TYPE, 
	Pdata_inicial IN votacio.data_inicial%TYPE, 
	Pdata_final IN votacio.data_final%TYPE, 
	Pdata_publicacio IN votacio.data_publicacio%TYPE, 
	Ppresident IN votacio.president%TYPE, 
	Pweb IN votacio.web%TYPE, 
	resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_votacio_update';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noCodi EXCEPTION;	
	noTitol EXCEPTION;
	noDescripcio EXCEPTION;
	noDataI EXCEPTION;
	noDataF EXCEPTION;
	noDataP EXCEPTION;
	noPresident EXCEPTION;	
	noTrobat EXCEPTION;
	noPot EXCEPTION;	
	altraExcepcio EXCEPTION;
	estatvot CHAR(1);
	err_num NUMBER;
	err_msg Varchar2(255);
BEGIN
        resultat := 'OK'; 
        entrada := '<'   || Pid_votacio  || '> <' || Ptitol || '> 
		   <' || Pdescripcio  || '> <' || Pdata_inicial || '>
		   <' || Pdata_final  || '> <' || Pdata_publicacio || '>
		   <' || Ppresident  || '> 
		   <' || Pweb || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Ptitol IS NULL OR Ptitol = ' ') THEN
            RAISE noTitol;
        END IF;
        IF (Pdescripcio IS NULL OR Pdescripcio = ' ') THEN
            RAISE noDescripcio;
        END IF;
        IF (Pdata_inicial IS NULL) THEN
            RAISE noDataI;
        END IF;
        IF (Pdata_final IS NULL) THEN
            RAISE noDataF;
        END IF;
        IF (Pdata_publicacio IS NULL) THEN
            RAISE noDataP;
        END IF;
        IF (Ppresident IS NULL OR Ppresident = ' ') THEN
            RAISE noPresident;
        END IF;
		-- Comprovem que la votacio existeixi
        BEGIN 
            SELECT estat INTO estatvot FROM Votacio
            WHERE Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
		IF estatvot <> 'E' THEN RAISE noPot;
		END IF;
        -- Update
        UPDATE Votacio SET 
			titol = Ptitol,
			descripcio = Pdescripcio,
			data_inicial = Pdata_inicial,
			data_final = Pdata_final,
			data_publicacio = Pdata_publicacio,
			president = Ppresident,
			web = Pweb
        WHERE id_votacio = Pid_votacio;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
		EXCEPTION
			WHEN noCodi THEN
			resultat := 'ERROR: <Codi de votaci> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobat THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTitol THEN
				resultat := 'ERROR: <Ttol> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDescripcio THEN
				resultat := 'ERROR: <Descripcio> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataI THEN
				resultat := 'ERROR: <Data inicial> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataF THEN
				resultat := 'ERROR: <Data final> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noDataP THEN
				resultat := 'ERROR: <Data de publicaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPresident THEN
				resultat := 'ERROR: <Ttol> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPot THEN
				resultat := 'ERROR: No es pot modificar votaci, no est en estat oberta';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_votacio_update;
/
--------------------------------------------------------------
-- Votaci - delete
-- Parmetres d'entrada: id_votacio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
--------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_votacio_delete (
    Pid_votacio IN votacio.id_votacio%TYPE,
	resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_votacio_delete';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(100);
	noTrobat EXCEPTION;
	altraExcepcio EXCEPTION;
	noCodi EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_votacio || '>';
        -- Comprobaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
		    RAISE noCodi;
        END IF;
        BEGIN 
            SELECT titol INTO nom FROM Votacio
            WHERE Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobat;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
        -- Eliminaci
        DELETE FROM Votacio
         WHERE id_votacio = Pid_votacio;
        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);
        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
			resultat := 'ERROR: <Codi de votaci> inexistent';
			p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobat THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_votacio_delete;
/


------------------------------------------------------
-- Opci - insert
-- Parmetres d'entrada: descripcio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_opcio_insert (
	Pdescripcio IN opcio.descripcio%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_opcio_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(50);
	err_num Number;
	err_msg Varchar2(255);
	noNom EXCEPTION;
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pdescripcio || '>';
        -- Comprovaci parmetres entrada
        IF (Pdescripcio IS NULL OR Pdescripcio = '') THEN
            RAISE noNom;
        END IF;
        
        -- Inserci a la taula
        INSERT INTO Opcio (descripcio)
                   VALUES (Pdescripcio);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noNom THEN
				resultat := 'ERROR: <Descripcio> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Opci ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_opcio_insert;
/
------------------------------------------------------
-- Opci - update
-- Parmetres d'entrada: id_opcio, descripcio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_opcio_update (
	Pid_opcio IN Opcio.id_opcio%TYPE, 
	Pdescripcio IN Opcio.descripcio%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_opcio_update';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(80);
	noOpcio EXCEPTION;
	noDesc EXCEPTION;
	noTrobat EXCEPTION;	
	altraExcepcio EXCEPTION;
	num_vots INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_opcio || '> <' || Pdescripcio || '> ';
        -- Comprovaci parmetres entrada
        IF (Pid_opcio IS NULL OR Pid_opcio = 0) THEN
            RAISE noOpcio;
        END IF;
        IF (Pdescripcio IS NULL OR Pdescripcio = ' ') THEN
            RAISE noDesc;
        END IF;
		-- Comprovaci que existeixi l'opcio
			BEGIN 
				SELECT descripcio INTO nom FROM Opcio
				WHERE Pid_opcio = id_opcio;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobat;
			WHEN OTHERS THEN
				RAISE altraExcepcio;
			END; 
        -- Update
        UPDATE Opcio set 
			descripcio = Pdescripcio
        WHERE id_opcio = Pid_opcio;
        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noDesc THEN
				resultat := 'ERROR: <Descripci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noOpcio THEN
				resultat := 'ERROR: <Codi opci> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: Lopci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_opcio_update;
/
------------------------------------------------------
-- Opci - delete
-- Parmetres d'entrada: id_opcio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_opcio_delete (
	Pid_opcio IN Opcio_Votacio.id_opcio%TYPE, 
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_opcio_delete';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(80);
	noOpcio EXCEPTION;
	noTrobat EXCEPTION;	
	noTrobaVot EXCEPTION;		
	estatvotacio CHAR(1);
	altraExcepcio EXCEPTION;
	num_vots INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_opcio || '> ';
        -- Comprovaci parmetres entrada
        IF (Pid_opcio IS NULL OR Pid_opcio = 0) THEN
            RAISE noOpcio;
        END IF;
		-- Comprovaci que existeixi l'opcio
		BEGIN 
			SELECT descripcio INTO nom FROM Opcio
			WHERE Pid_opcio = id_opcio;
		EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobat;
		WHEN OTHERS THEN
			RAISE altraExcepcio;
		END; 
		-- Esborra l'opci
		DELETE FROM Opcio
		WHERE id_opcio = Pid_opcio;
        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noOpcio THEN
				resultat := 'ERROR: <Codi opci> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: Lopci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_opcio_delete;
/


------------------------------------------------------
-- Opci Votaci - insert
-- Parmetres d'entrada: id_opcio, id_votacio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_opcio_votacio_insert (
	Pid_opcio IN Opcio_Votacio.id_opcio%TYPE, 
	Pid_votacio IN Opcio_Votacio.id_votacio%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_opcio_votacio_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(80);
	noOp EXCEPTION;
	noVot EXCEPTION;
	noTrobaVot EXCEPTION;
	noTrobaOp EXCEPTION;	
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_opcio || '> <' || Pid_votacio || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_opcio IS NULL OR Pid_opcio = 0) THEN
            RAISE noOp;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;

        BEGIN 
            SELECT titol INTO nom FROM Votacio
            WHERE Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
        BEGIN 
            SELECT descripcio INTO nom FROM Opcio
            WHERE Pid_opcio = id_opcio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaOp;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 

        -- Inserci a la taula
        INSERT INTO Opcio_Votacio (id_opcio, id_votacio, num_vots_opcio, perc_sobre_total)
                   VALUES (Pid_opcio, Pid_votacio, 0, 0);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noOp THEN
				resultat := 'ERROR: <Codi opci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaOp THEN
				resultat := 'ERROR: Opci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Opci_Votaci ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_opcio_votacio_insert;
/

------------------------------------------------------
-- Opci Votaci - update
-- Parmetres d'entrada: id_opcio, id_votacio, 
--						num_vots_opcio, perc_sobre_total
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_opcio_votacio_update (
	Pid_opcio IN Opcio_Votacio.id_opcio%TYPE, 
	Pid_votacio IN Opcio_Votacio.id_votacio%TYPE,
	Pnum_vots_opcio IN Opcio_Votacio.num_vots_opcio%TYPE,
	Pperc_sobre_total IN Opcio_Votacio.perc_sobre_total%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_opcio_votacio_update';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(80);
	noOpcio EXCEPTION;
	noCodi EXCEPTION;
	noTrobat EXCEPTION;	
	altraExcepcio EXCEPTION;
	num_vots INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_opcio || '> <' || Pid_votacio || '> <' || Pnum_vots_opcio || '> 
					<' || Pperc_sobre_total || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_opcio IS NULL OR Pid_opcio = 0) THEN
            RAISE noOpcio;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noCodi;
        END IF;
		-- Comprovaci que existeixi opcio_votacio
			BEGIN 
				SELECT num_vots_opcio INTO num_vots FROM opcio_votacio
				WHERE Pid_opcio = id_opcio
				AND Pid_votacio = id_votacio;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				RAISE noTrobat;
			WHEN OTHERS THEN
				RAISE altraExcepcio;
			END; 
	
        -- Update
        UPDATE opcio_votacio set 
			num_vots_opcio = Pnum_vots_opcio,
			perc_sobre_total = Pperc_sobre_total
        WHERE id_opcio = Pid_opcio
		AND id_votacio = Pid_votacio;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noOpcio THEN
				resultat := 'ERROR: <Codi opci> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: La Opci_Votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_opcio_votacio_update;
/
------------------------------------------------------
-- Opci Votaci - delete
-- Parmetres d'entrada: id_opcio, id_votacio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_opcio_votacio_delete (
	Pid_opcio IN Opcio_Votacio.id_opcio%TYPE, 
	Pid_votacio IN Opcio_Votacio.id_votacio%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_opcio_votacio_delete';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(80);
	noOpcio EXCEPTION;
	noCodi EXCEPTION;
	noTrobat EXCEPTION;	
	noPot EXCEPTION;	
	noTrobaVot EXCEPTION;		
	estatvotacio CHAR(1);
	altraExcepcio EXCEPTION;
	num_vots INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_opcio || '> <' || Pid_votacio || '> ';
        -- Comprovaci parmetres entrada
        IF (Pid_opcio IS NULL OR Pid_opcio = 0) THEN
            RAISE noOpcio;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noCodi;
        END IF;
		-- Comprovaci que existeixi opcio_votacio
		BEGIN 
			SELECT num_vots_opcio INTO num_vots FROM opcio_votacio
			WHERE Pid_opcio = id_opcio
			AND Pid_votacio = id_votacio;
		EXCEPTION
		WHEN NO_DATA_FOUND THEN
			RAISE noTrobat;
		WHEN OTHERS THEN
			RAISE altraExcepcio;
		END; 
		-- Comprovaci de l'estat de la votaci
        BEGIN 
            SELECT estat INTO estatvotacio FROM Votacio
            WHERE Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		IF estatvotacio = 'O' OR estatvotacio = 'F' THEN RAISE noPot;
		END IF;
		-- Esborra l'opci-votaci
		DELETE FROM Opcio_Votacio
		WHERE id_votacio = Pid_votacio
		AND id_opcio = Pid_opcio;

        -- Insert al Log
		p_log_insert(procediment, entrada, sortida, resultat);

        -- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noOpcio THEN
				resultat := 'ERROR: <Codi opci> inexistent';				
				p_log_insert(procediment, entrada, sortida, resultat);				
			WHEN noTrobat THEN
				resultat := 'ERROR: La Opci_Votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPot THEN
				resultat := 'ERROR: La votacio est oberta o finalitzada';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_opcio_votacio_delete;
/


------------------------------------------------------
-- Vota - insert
-- Parmetres d'entrada: codi_ciutada, id_votacio, id_opcio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_vota_insert (
	Pcodi_ciutada IN Vota.codi_ciutada%TYPE, 
	Pid_votacio IN Vota.id_votacio%TYPE,
	Pid_opcio IN Vota.id_opcio%TYPE,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_vota_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(80);
	localitat INTEGER;
	numvots INTEGER;
	ciutada INTEGER;
	estatvotacio CHAR(1);	
	noCodi EXCEPTION;	
	noOp EXCEPTION;
	noVot EXCEPTION;
	noTrobaVot EXCEPTION;
	noTrobaOp EXCEPTION;	
	noTrobaCiu EXCEPTION;
	noTrobaAss EXCEPTION;
	noOberta EXCEPTION;		
	noTrobaOpVot EXCEPTION;
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pcodi_ciutada || '> <'|| Pid_votacio || '> <' || Pid_opcio || '>';
        -- Comprovaci parmetres entrada
        IF (Pcodi_ciutada IS NULL OR Pcodi_ciutada = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pid_opcio IS NULL OR Pid_opcio = 0) THEN
            RAISE noOp;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;

        BEGIN 
            SELECT estat INTO estatvotacio FROM Votacio
            WHERE Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		IF estatvotacio <> 'O' THEN RAISE noOberta;
		END IF;
        BEGIN 
            SELECT descripcio INTO nom FROM Opcio
            WHERE Pid_opcio = id_opcio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaOp;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
        BEGIN 
            SELECT id_localitat INTO localitat FROM Ciutada
            WHERE Pcodi_ciutada = codi_ciutada;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaCiu;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
        BEGIN 
            SELECT num_vots_opcio INTO numvots FROM Opcio_Votacio
            WHERE Pid_opcio = id_opcio AND Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaOpVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
		-- Comprova que el ciutad estigui assignat per votar
        BEGIN 
            SELECT codi_ciutada INTO ciutada FROM Assignacio
            WHERE Pid_votacio = id_votacio AND Pcodi_ciutada = codi_ciutada;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaAss;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 

        -- Inserci a la taula
        INSERT INTO Vota (codi_ciutada, id_votacio, id_opcio, data_hora_vot)
                   VALUES (Pcodi_ciutada, Pid_votacio, Pid_opcio, SYSDATE);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noOp THEN
				resultat := 'ERROR: <Codi opci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaOp THEN
				resultat := 'ERROR: <Codi opci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaCiu THEN
				resultat := 'ERROR: <Codi ciutad> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaAss THEN
				resultat := 'ERROR: Ciutad no est cridat a votar';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaOpVot THEN
				resultat := 'ERROR: Opci no permesa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noOberta THEN
				resultat := 'ERROR: La votaci no est oberta';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El ciutad ja ha votat';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_vota_insert;
/
------------------------------------------------------
-- Assignaci - insert
-- Parmetres d'entrada: id_votacio, codi_ciutada, id_cens
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_assignacio_insert (
	Pid_votacio IN assignacio.id_votacio%TYPE,
	Pcodi_ciutada IN assignacio.codi_ciutada%TYPE,	
	Pid_cens IN assignacio.id_cens%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_assignacio_insert';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noCodi EXCEPTION;	
	noCens EXCEPTION;
	noVot EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_votacio || '> <'|| Pcodi_ciutada || '> <'|| Pid_cens || '>';
        -- Comprovaci parmetres entrada
        IF (Pcodi_ciutada IS NULL OR Pcodi_ciutada = 0) THEN
            RAISE noCodi;
        END IF;
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCens;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;

        -- Inserci a la taula
        INSERT INTO Assignacio (id_votacio, codi_ciutada, id_cens, data_assignacio)
                   VALUES (Pid_votacio, Pcodi_ciutada, Pid_cens, SYSDATE);
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Codi ciutada> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCens THEN
				resultat := 'ERROR: <Codi cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: Assignaci ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_assignacio_insert;
/
------------------------------------------------------
-- Assignacio Global
-- Parmetres d'entrada: id_votacio, id_cens
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_assignacio_global (
	Pid_votacio IN assignacio.id_votacio%TYPE,
	Pid_cens IN assignacio.id_cens%TYPE,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_assignacio_global';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	edat INTEGER;
	votant INTEGER;	
	codi INTEGER;		
	noVot EXCEPTION;
	noTrobaCens EXCEPTION;	
	noTrobaVot EXCEPTION;	
	noCens EXCEPTION;		
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
	RSP Varchar2(100);
	datavotacio DATE;
	datanaixvot DATE;
	-- Definim el cursor
	CURSOR c_assignacio_global IS
		SELECT codi_ciutada INTO codi FROM Ciutada
		WHERE (data_naixament <= datanaixvot) AND data_baixa IS NULL;
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
		RSP := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pid_cens || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCens;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
		-- Comprovaci de que el cens existeixi
        BEGIN 
            SELECT edat_minima INTO edat FROM CensGlobal
            WHERE Pid_cens = id_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaCens;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Obtenim la data inicial de la votaci
        BEGIN 
            SELECT data_inicial INTO datavotacio FROM Votacio
            WHERE Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Calculem la data de naixament mxima per poder votar
		datanaixvot := add_months(datavotacio,-edat*12);
		-- Assignem tots els ciutadans que tenen dret a vot

		BEGIN
			-- Tractament del cursor
			OPEN c_assignacio_global;
			LOOP
				FETCH c_assignacio_global INTO votant;
				EXIT WHEN c_assignacio_global%NOTFOUND;
				p_assignacio_insert(Pid_votacio, votant, Pid_cens, RSP);
			END LOOP;
			CLOSE c_assignacio_global;
		END;

		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCens THEN
				resultat := 'ERROR: <Codi cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaCens THEN
				resultat := 'ERROR: El cens no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: la votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_assignacio_global;
/
------------------------------------------------------
-- Assignacio Estatal
-- Parmetres d'entrada: id_votacio, id_cens
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_assignacio_estatal (
	Pid_votacio IN assignacio.id_votacio%TYPE,
	Pid_cens IN assignacio.id_cens%TYPE,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_assignacio_estatal';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	edat INTEGER;
	votant INTEGER;	
	codi INTEGER;	
	codipais CHAR(2);		
	noVot EXCEPTION;
	noTrobaCens EXCEPTION;	
	noTrobaVot EXCEPTION;	
	noCens EXCEPTION;		
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
	RSP Varchar2(100);
	datavotacio DATE;
	datanaixvot DATE;
	-- Definim el cursor
	CURSOR c_assignacio_estatal IS
		SELECT codi_ciutada INTO codi FROM Ciutada
		WHERE id_localitat IN 
			(SELECT id_localitat FROM Localitat WHERE id_regio IN
					(SELECT id_regio FROM Regio WHERE codi_pais IN
						(SELECT codi_pais FROM Pais WHERE codi_pais = codipais)
					)
			)		
		AND (data_naixament <= datanaixvot) 
		AND (data_baixa IS NULL);
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
		RSP := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pid_cens || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCens;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
		-- Comprovaci de que el cens existeixi
        BEGIN 
            SELECT codi_pais INTO codipais FROM CensEstatal
            WHERE Pid_cens = id_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaCens;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Obtenim la data inicial de la votaci
        BEGIN 
            SELECT data_inicial INTO datavotacio FROM Votacio
            WHERE Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Calculem la data de naixament mxima per poder votar
		-- Considerem que l'edat per votar s de 18 anys
		edat := 18; 
		datanaixvot := add_months(datavotacio,-edat*12);
		-- Assignem tots els ciutadans que tenen dret a vot

		BEGIN
			-- Tractament del cursor
			OPEN c_assignacio_estatal;
			LOOP
				FETCH c_assignacio_estatal INTO votant;
				EXIT WHEN c_assignacio_estatal%NOTFOUND;
				p_assignacio_insert(Pid_votacio, votant, Pid_cens, RSP);
			END LOOP;
			CLOSE c_assignacio_estatal;
		END;

		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCens THEN
				resultat := 'ERROR: <Codi cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaCens THEN
				resultat := 'ERROR: El cens no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: la votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_assignacio_estatal;
/
------------------------------------------------------
-- Assignacio Local
-- Parmetres d'entrada: id_votacio, id_cens
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_assignacio_local (
	Pid_votacio IN assignacio.id_votacio%TYPE,
	Pid_cens IN assignacio.id_cens%TYPE,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_assignacio_local';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	edat INTEGER;
	votant INTEGER;	
	codiloc INTEGER;		
	codi INTEGER;		
	noVot EXCEPTION;
	noTrobaCens EXCEPTION;	
	noTrobaVot EXCEPTION;	
	noCens EXCEPTION;		
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
	RSP Varchar2(100);
	datavotacio DATE;
	datanaixvot DATE;
	-- Definim el cursor
	CURSOR c_assignacio_local IS
		SELECT codi_ciutada INTO codi FROM Ciutada
		WHERE (data_naixament <= datanaixvot) 
		AND (data_baixa IS NULL)
		AND (id_localitat = codiloc);
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
		RSP := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pid_cens || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCens;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
		-- Comprovaci de que el cens existeixi
        BEGIN 
            SELECT id_localitat INTO codiloc FROM CensLocal
            WHERE Pid_cens = id_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaCens;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Obtenim la data inicial de la votaci
        BEGIN 
            SELECT data_inicial INTO datavotacio FROM Votacio
            WHERE Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Calculem la data de naixament mxima per poder votar
		-- Considerem que l'edat per votar s de 18 anys
		edat := 18; 
		datanaixvot := add_months(datavotacio,-edat*12);
		-- Assignem tots els ciutadans que tenen dret a vot

		BEGIN
			-- Tractament del cursor
			OPEN c_assignacio_local;
			LOOP
				FETCH c_assignacio_local INTO votant;
				EXIT WHEN c_assignacio_local%NOTFOUND;
				p_assignacio_insert(Pid_votacio, votant, Pid_cens, RSP);
			END LOOP;
			CLOSE c_assignacio_local;
		END;

		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCens THEN
				resultat := 'ERROR: <Codi cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaCens THEN
				resultat := 'ERROR: El cens no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: la votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_assignacio_local;
/
------------------------------------------------------
-- Assignacio Regional
-- Parmetres d'entrada: id_votacio, id_cens
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_assignacio_regional (
	Pid_votacio IN assignacio.id_votacio%TYPE,
	Pid_cens IN assignacio.id_cens%TYPE,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_assignacio_regional';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	edat INTEGER;
	votant INTEGER;	
	codireg INTEGER;		
	codi INTEGER;		
	noVot EXCEPTION;
	noTrobaCens EXCEPTION;	
	noTrobaVot EXCEPTION;	
	noCens EXCEPTION;		
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
	RSP Varchar2(100);
	datavotacio DATE;
	datanaixvot DATE;
	-- Definim el cursor
	CURSOR c_assignacio_regional IS
		SELECT codi_ciutada INTO codi FROM Ciutada
		WHERE id_localitat IN 
			(SELECT id_localitat FROM Localitat WHERE id_regio IN
					(SELECT id_regio FROM Regio WHERE id_regio = codireg)
			)		
		AND (data_naixament <= datanaixvot) 
		AND (data_baixa IS NULL);
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
		RSP := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pid_cens || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCens;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
		-- Comprovaci de que el cens existeixi
        BEGIN 
            SELECT id_regio INTO codireg FROM CensRegional
            WHERE Pid_cens = id_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaCens;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Obtenim la data inicial de la votaci
        BEGIN 
            SELECT data_inicial INTO datavotacio FROM Votacio
            WHERE Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Calculem la data de naixament mxima per poder votar
		-- Considerem que l'edat per votar s de 18 anys
		edat := 18; 
		datanaixvot := add_months(datavotacio,-edat*12);
		-- Assignem tots els ciutadans que tenen dret a vot

		BEGIN
			-- Tractament del cursor
			OPEN c_assignacio_regional;
			LOOP
				FETCH c_assignacio_regional INTO votant;
				EXIT WHEN c_assignacio_regional%NOTFOUND;
				p_assignacio_insert(Pid_votacio, votant, Pid_cens, RSP);
			END LOOP;
			CLOSE c_assignacio_regional;
		END;

		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCens THEN
				resultat := 'ERROR: <Codi cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaCens THEN
				resultat := 'ERROR: El cens no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: la votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_assignacio_regional;
/
------------------------------------------------------
-- Assignacio General
-- Parmetres d'entrada: id_votacio, id_cens
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_assignacio (
	Pid_votacio IN assignacio.id_votacio%TYPE,
	Pid_cens IN assignacio.id_cens%TYPE,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_assignacio';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(80);
	tipus CHAR(1);
	numvots INTEGER;
	ciutada INTEGER;
	estatvotacio CHAR(1);	
	noOp EXCEPTION;
	noVot EXCEPTION;
	noCens EXCEPTION;	
	noTipus EXCEPTION;	
	noTrobaVot EXCEPTION;
	noTrobaCens EXCEPTION;	
	noTrobaCiu EXCEPTION;
	noPot EXCEPTION;		
	noTrobaOpVot EXCEPTION;
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
	RSP Varchar2(100);
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pid_cens || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCens;
        END IF;
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;

        BEGIN 
            SELECT estat INTO estatvotacio FROM Votacio
            WHERE Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		IF estatvotacio = 'O' OR estatvotacio = 'F' THEN RAISE noPot;
		END IF;
        BEGIN 
            SELECT tipus_cens INTO tipus FROM Cens
            WHERE Pid_cens = id_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaCens;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END; 
		-- Depenent del tipus de cens fa una assignaci global, estatal, regional o local
		CASE tipus
		WHEN 'G' THEN p_assignacio_global (Pid_votacio, Pid_cens, RSP);
		WHEN 'E' THEN p_assignacio_estatal (Pid_votacio, Pid_cens, RSP);
		WHEN 'R' THEN p_assignacio_regional (Pid_votacio, Pid_cens, RSP);
		WHEN 'L' THEN p_assignacio_local (Pid_votacio, Pid_cens, RSP);
		ELSE RAISE noTipus;
		END CASE;

        resultat := RSP;
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCens THEN
				resultat := 'ERROR: <Codi cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaCens THEN
				resultat := 'ERROR: El cens no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaCiu THEN
				resultat := 'ERROR: <Codi ciutad> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaOpVot THEN
				resultat := 'ERROR: Opci no permesa';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTipus THEN
				resultat := 'ERROR: Tipus no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPot THEN
				resultat := 'ERROR: La votaci no est en edici ni pendent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_assignacio;
/

-----------------------------------------------------------
-- Assignacio General - esborrar l'assignaci d'una votaci
-- Parmetres d'entrada: id_votacio
-- Parmetres de sortida: resultat (RSP)
-- Procediments que executa: p_log_insert
-----------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_assignacio_delete (
	Pid_votacio IN assignacio.id_votacio%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_assignacio_delete';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	nom Varchar2(80);
	tipus CHAR(1);
	numvots INTEGER;
	ciutada INTEGER;
	estatvotacio CHAR(1);	
	noOp EXCEPTION;
	noVot EXCEPTION;
	noCens EXCEPTION;	
	noTipus EXCEPTION;	
	noTrobaVot EXCEPTION;
	noTrobaCens EXCEPTION;	
	noTrobaCiu EXCEPTION;
	noPot EXCEPTION;		
	noTrobaOpVot EXCEPTION;
	altraExcepcio EXCEPTION;
	err_num NUMBER;
	err_msg Varchar2(255);
	RSP Varchar2(100);
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_votacio || '> ';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
		-- Comprovaci de l'estat de la votaci
        BEGIN 
            SELECT estat INTO estatvotacio FROM Votacio
            WHERE Pid_votacio = id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		IF estatvotacio = 'O' OR estatvotacio = 'F' THEN RAISE noPot;
		END IF;
		-- Esborra els registres de la votaci dins de la taula assignacio
		DELETE FROM Assignacio
		WHERE id_votacio = Pid_votacio;
     
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTipus THEN
				resultat := 'ERROR: Tipus no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPot THEN
				resultat := 'ERROR: La votaci no est en edici ni pendent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_assignacio_delete;
/
