----------------------------------------------------------------------------
-- Llistats - p_votacions_pais_llistat  
-- 		Donat un pas, el llistat de totes les votacions que s'hi han produt
-- 		(que han tingut assignat algun cens pertanyent a aquell pas o al 
--		cens global)
--
-- Parmetres d'entrada: codi_pais
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_votacions_pais_llistat (
	Pcodi_pais IN Pais.codi_pais%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_votacions_pais_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noPais EXCEPTION;
	noPais2 EXCEPTION;	
	altraExcepcio EXCEPTION;
	nomtitol Varchar2(100);
	nom Varchar2(100);	
	codi INTEGER;
	codivot INTEGER;	
	dataini DATE;
	datafi DATE;
	estat CHAR(1);
	nomestat Varchar2(30);
	codicens INTEGER;
	perc_par NUMBER(5,2);
	guanyador INTEGER;
	nom_guanyador Varchar2(100);
	perc_guanyador NUMBER(5,2);
	num_censats INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
	CURSOR c_pais IS
		select distinct v.id_votacio, v.titol, v.data_inicial, v.data_final, 
		v.num_pers_capacitat_vot, v.estat, a.id_cens, 
		v.perc_participacio, v.id_opcio_guanyadora, v.perc_vots_opcio_guanyadora
		from votacio v, assignacio a, censglobal c, censestatal e 
		where v.id_votacio = a.id_votacio
		and ((a.id_cens = c.id_cens) or 
		((e.id_cens = a.id_cens) and (e.codi_pais = Pcodi_pais)))
		order by data_inicial asc;
	BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '< ' || Pcodi_pais || ' >';
		-- Comprova parmetres d'entrada
        IF (Pcodi_pais IS NULL OR Pcodi_pais = ' ') THEN
            RAISE noPais;
        END IF;
		-- Comprova que existeixi el pas
        BEGIN 
            SELECT nom_pais INTO nom
			FROM Pais
            WHERE codi_pais = Pcodi_pais;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noPais2;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Llistat de votacions que shan produt en un pas');
		DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');
		DBMS_OUTPUT.PUT_LINE('Codi de pas: '||Pcodi_pais);
		DBMS_OUTPUT.PUT_LINE('Nom pas: '||nom);		
		DBMS_OUTPUT.PUT_LINE(' ');
		-- Llistem les votacions que ha participat el ciutad
		DBMS_OUTPUT.PUT_LINE('Votacions');
		DBMS_OUTPUT.PUT_LINE('---------');
		BEGIN
			-- Tractament del cursor
			OPEN c_pais;
			LOOP
				FETCH c_pais INTO codivot, nomtitol, dataini, datafi, num_censats, estat, 
					codicens, perc_par, guanyador, perc_guanyador;	
				EXIT WHEN c_pais%NOTFOUND;
				DBMS_OUTPUT.PUT_LINE('Codi votaci: '||codivot);						
				DBMS_OUTPUT.PUT_LINE('Titol: '||nomtitol);		
				DBMS_OUTPUT.PUT_LINE('Data inicial: '||to_char(dataini,'DD/MM/YY'));						
				DBMS_OUTPUT.PUT_LINE('Data final: '||to_char(datafi,'DD/MM/YY'));
				DBMS_OUTPUT.PUT_LINE('Ciutadans censats: '||num_censats);						
				IF estat = 'O' THEN nomestat := 'Oberta';
				END IF;
				IF estat = 'E' THEN nomestat := 'En edici';
				END IF;
				IF estat = 'P' THEN nomestat := 'Pendent';
				END IF;
				IF estat = 'F' THEN nomestat := 'Finalitzada';
				END IF;
				DBMS_OUTPUT.PUT_LINE('Estat: '||nomestat);
				DBMS_OUTPUT.PUT_LINE('Codi cens: '||codicens);
				IF estat = 'F' THEN
					nom_guanyador := ' ';
					SELECT descripcio INTO nom_guanyador
					FROM Opcio
					WHERE id_opcio = guanyador;
					DBMS_OUTPUT.PUT_LINE('Percentatge participaci: '||perc_par||' %');
					DBMS_OUTPUT.PUT_LINE('Opci guanyadora: '||nom_guanyador);
					DBMS_OUTPUT.PUT_LINE('Percentatge opci guanyadora: '||perc_guanyador||' %');
				ELSE
					DBMS_OUTPUT.PUT_LINE('Percentatge participaci: -');
					DBMS_OUTPUT.PUT_LINE('Opci guanyadora: -');
					DBMS_OUTPUT.PUT_LINE('Percentatge opci guanyadora: -');
				END IF;
				DBMS_OUTPUT.PUT_LINE('---------------------------------------------------------------');
			END LOOP;
			CLOSE c_pais;
		END;
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
		-- Control d'excepcions
        EXCEPTION
			WHEN noPais THEN
				resultat := 'ERROR: <Codi pas> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais2 THEN
				resultat := 'ERROR: El pas no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_votacions_pais_llistat;
/

----------------------------------------------------------------------------
-- Llistats - p_10votacions_mesdif_llistat 
-- 		10 votacions on hi ha hagut ms diferncia percentual de vots entre 
-- 		l'opci ms votada i la menys votada
--
-- Parmetres d'entrada: 
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_10votacions_mesdif_llistat (
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_10votacions_mesdif_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noAny EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	perc_maxim NUMBER(5,2);
	perc_minim NUMBER(5,2);	
	diferencia NUMBER(5,2);		
	nomtitol Varchar2(100);
	data1 Varchar2(12);
	data2 Varchar2(12);
	guanyador Varchar2(100);
	err_num NUMBER;
	err_msg Varchar2(255);	
	CURSOR c_10votacions IS
	-- Calcula les opcions i els mxims i mnims percentuals
		select v.titol, to_char(v.data_inicial,'DD/MM/YY') as dataini, 
		to_char(v.data_final,'DD/MM/YY') as datafi, 
		o.descripcio as guanyador, 
		max(op.perc_sobre_total) as maxim, min(op.perc_sobre_total) as minim
		from opcio_votacio op, votacio v, opcio o
		where v.id_votacio = op.id_votacio
		and v.id_opcio_guanyadora = o.id_opcio
		group by v.titol, v.data_inicial, v.data_final, o.descripcio
		order by (maxim-minim) desc;
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '< Sense parmetres >';
		-- Tractament dels cursors per obtenir els totals amb el RowCount
		-- Total de no votants
		BEGIN
			-- Tractament del cursor
			diferencia := 0;
			DBMS_OUTPUT.PUT_LINE('Llistat 10 votacions ms diferncia percentual de vots ');
			DBMS_OUTPUT.PUT_LINE('-------------------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
			DBMS_OUTPUT.PUT_LINE(RPAD('Ttol de la votaci',40)||RPAD('  ',2)||RPAD('Data inicial',14)||RPAD('Data final',14)||
				RPAD('Opci ms votada',40)||RPAD('  ',2)||RPAD('Perc. mxim',14)||RPAD('Perc. mnim',14)||RPAD('Diferncia',14));
			DBMS_OUTPUT.PUT_LINE(RPAD('-------------------',40)||RPAD('  ',2)||RPAD('------------',14)||RPAD('----------',14)||
				RPAD('----------------',40)||RPAD('  ',2)||RPAD('-----------',14)||RPAD('-----------',14)||RPAD('----------',14));				
			OPEN c_10votacions;
			LOOP
				FETCH c_10votacions INTO nomtitol, data1, data2, guanyador, perc_maxim, perc_minim;
				EXIT WHEN c_10votacions%NOTFOUND OR c_10votacions%ROWCOUNT=11;
				diferencia := perc_maxim - perc_minim;
				DBMS_OUTPUT.PUT_LINE(RPAD(nomtitol,40)||RPAD('  ',2)||RPAD(data1,14)||RPAD(data2,14)||RPAD(guanyador,40)
									||RPAD('  ',2)||RPAD(perc_maxim,14)||RPAD(perc_minim,14)||RPAD(diferencia,14));				
			END LOOP;
			CLOSE c_10votacions;
		END;
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E7 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_10votacions_mesdif_llistat;
/
----------------------------------------------------------------------------
-- Llistats - p_10votacions_menysdif_llistat 
-- 		10 votacions on hi ha hagut menys diferncia percentual de vots entre 
-- 		l'opci ms votada i la menys votada
--
-- Parmetres d'entrada: 
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_10votacions_menysdif_llistat (
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_10votacions_menysdif_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noAny EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	perc_maxim NUMBER(5,2);
	perc_minim NUMBER(5,2);	
	diferencia NUMBER(5,2);		
	nomtitol Varchar2(100);
	data1 Varchar2(12);
	data2 Varchar2(12);
	guanyador Varchar2(100);
	err_num NUMBER;
	err_msg Varchar2(255);	
	CURSOR c_10votacions IS
	-- Calcula les opcions i els mxims i mnims percentuals
		select v.titol, to_char(v.data_inicial,'DD/MM/YY') as dataini, 
		to_char(v.data_final,'DD/MM/YY') as datafi, 
		o.descripcio as guanyador, 
		max(op.perc_sobre_total) as maxim, min(op.perc_sobre_total) as minim
		from opcio_votacio op, votacio v, opcio o
		where v.id_votacio = op.id_votacio
		and v.id_opcio_guanyadora = o.id_opcio
		group by v.titol, v.data_inicial, v.data_final, o.descripcio
		order by (maxim-minim) asc;
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '< Sense parmetres >';
		BEGIN
			-- Tractament del cursor
			diferencia := 0;
			DBMS_OUTPUT.PUT_LINE('Llistat 10 votacions menys diferncia percentual de vots ');
			DBMS_OUTPUT.PUT_LINE('---------------------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
			DBMS_OUTPUT.PUT_LINE(RPAD('Ttol de la votaci',40)||RPAD('  ',2)||RPAD('Data inicial',14)||RPAD('Data final',14)||
				RPAD('Opci ms votada',40)||RPAD('  ',2)||RPAD('Perc. mxim',14)||RPAD('Perc. mnim',14)||RPAD('Diferncia',14));
			DBMS_OUTPUT.PUT_LINE(RPAD('-------------------',40)||RPAD('  ',2)||RPAD('------------',14)||RPAD('----------',14)||
				RPAD('----------------',40)||RPAD('  ',2)||RPAD('-----------',14)||RPAD('-----------',14)||RPAD('----------',14));				
			OPEN c_10votacions;
			LOOP
				FETCH c_10votacions INTO nomtitol, data1, data2, guanyador, perc_maxim, perc_minim;
				EXIT WHEN c_10votacions%NOTFOUND OR c_10votacions%ROWCOUNT=11;
				diferencia := perc_maxim - perc_minim;
				DBMS_OUTPUT.PUT_LINE(RPAD(nomtitol,40)||RPAD('  ',2)||RPAD(data1,14)||RPAD(data2,14)||RPAD(guanyador,40)
									||RPAD('  ',2)||RPAD(perc_maxim,14)||RPAD(perc_minim,14)||RPAD(diferencia,14));				
			END LOOP;
			CLOSE c_10votacions;
		END;
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E7 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_10votacions_menysdif_llistat;
/

----------------------------------------------------------------------------
-- Llistats - p_censos_ciutada_llistat 
-- 		Donat el codi aleatori que identifica un ciutad, els censos
-- 		als que pertany
--
-- Parmetres d'entrada: codi_ciutada
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_censos_ciutada_llistat (
	Pcodi_ciutada IN Ciutada.codi_ciutada%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_censos_ciutada_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noCiu EXCEPTION;
	noCodi EXCEPTION;	
	altraExcepcio EXCEPTION;
	nomtitol Varchar2(100);
	nom Varchar2(100);	
	codi INTEGER;
	codicens INTEGER;	
	edat INTEGER;
	total INTEGER;
	camp_loc INTEGER;
	camp_regio INTEGER;
	camp_pais CHAR(2);
	err_num NUMBER;
	err_msg Varchar2(255);	
	BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '< ' ||Pcodi_ciutada|| ' >';
		-- Comprova parmetres d'entrada
        IF (Pcodi_ciutada IS NULL OR Pcodi_ciutada = 0) THEN
            RAISE noCiu;
        END IF;
		-- Comprova que existeixi el ciutad
        BEGIN 
            SELECT codi_ciutada INTO codi
			FROM Ciutada
            WHERE codi_ciutada = Pcodi_ciutada;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noCiu;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
	-- Busca la localitat, regi i pas del ciutad
        BEGIN 
			select c.codi_ciutada, l.id_localitat, r.id_regio, p.codi_pais 
					INTO codi, camp_loc, camp_regio, camp_pais
			from ciutada c, localitat l, regio r, pais p
			where c.id_localitat = l.id_localitat
			and l.id_regio = r.id_regio
			and r.codi_pais = p.codi_pais
			and c.codi_ciutada = Pcodi_ciutada;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noCiu;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Llistat de censos que pertnyen a un ciutad ');
		DBMS_OUTPUT.PUT_LINE('---------------------------------------------');
		DBMS_OUTPUT.PUT_LINE('Codi de ciutad: '||Pcodi_ciutada);
		DBMS_OUTPUT.PUT_LINE('Codi de localitat: '||camp_loc);			
		DBMS_OUTPUT.PUT_LINE('Codi de regi: '||camp_regio);			
		DBMS_OUTPUT.PUT_LINE('Codi de pas: '||camp_pais);
		DBMS_OUTPUT.PUT_LINE(' ');
		-- Comprovem els cens globals
		codicens := 0;
		BEGIN
			SELECT id_cens, edat_minima INTO codicens, edat 
			FROM censglobal;
			IF codicens > 0 THEN
				DBMS_OUTPUT.PUT_LINE('Codi cens global: '||codicens);
				DBMS_OUTPUT.PUT_LINE('Edat mnima: '||edat);
				DBMS_OUTPUT.PUT_LINE(' ');			
			END IF;
		END;
		-- Comprovem els cens estatals d'aquell ciutad
		codicens := 0;
		BEGIN
			SELECT c.id_cens, p.nom_pais INTO codicens, nom
			FROM censestatal c, pais p
			WHERE c.codi_pais = camp_pais
			AND c.codi_pais = p.codi_pais;
			IF codicens > 0 THEN
				DBMS_OUTPUT.PUT_LINE('Codi cens estatal: '||codicens);
				DBMS_OUTPUT.PUT_LINE('Pas: '||nom);
				DBMS_OUTPUT.PUT_LINE(' ');			
			END IF;
		END;
		-- Comprovem els cens regionals d'aquell ciutad
		codicens := 0;
		BEGIN
			SELECT c.id_cens, r.nom_regio INTO codicens, nom
			FROM censregional c, regio r
			WHERE r.id_regio = camp_regio
			AND c.id_regio = r.id_regio;
			IF codicens > 0 THEN
				DBMS_OUTPUT.PUT_LINE('Codi cens regional: '||codicens);
				DBMS_OUTPUT.PUT_LINE('Regi: '||nom);
				DBMS_OUTPUT.PUT_LINE(' ');			
			END IF;
		END;
		-- Comprovem els cens locals d'aquell ciutad
		codicens := 0;
		BEGIN
			SELECT c.id_cens, l.nom_localitat INTO codicens, nom
			FROM censlocal c, localitat l
			WHERE l.id_localitat = camp_loc
			AND c.id_localitat = l.id_localitat;
			IF codicens > 0 THEN
				DBMS_OUTPUT.PUT_LINE('Codi cens local: '||codicens);
				DBMS_OUTPUT.PUT_LINE('Localitat: '||nom);
				DBMS_OUTPUT.PUT_LINE(' ');			
			END IF;
		END;
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
		-- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Ciutad> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCiu THEN
				resultat := 'ERROR: El ciutad no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_censos_ciutada_llistat;
/

----------------------------------------------------------------------------
-- Llistats - p_votacions_ciutada_llistat 
-- 		Donat el codi aleatori que identifica un ciutad, les votacions
-- 		que ha participat i les que no
--
-- Parmetres d'entrada: codi_ciutada
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_votacions_ciutada_llistat (
	Pcodi_ciutada IN Ciutada.codi_ciutada%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_votacions_ciutada_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noCiu EXCEPTION;
	noCodi EXCEPTION;	
	altraExcepcio EXCEPTION;
	nomtitol Varchar2(100);
	nom Varchar2(100);	
	codi INTEGER;
	codivot INTEGER;	
	edat INTEGER;
	total INTEGER;
	camp_loc INTEGER;
	camp_regio INTEGER;
	camp_pais CHAR(2);
	err_num NUMBER;
	err_msg Varchar2(255);	
	CURSOR c_vots IS
		select v.id_votacio, vo.titol
		from vota v, votacio vo
		where v.codi_ciutada = Pcodi_ciutada
		and v.id_votacio = vo.id_votacio;
	CURSOR c_vots2 IS
		select a.id_votacio, v.titol
		from assignacio a, votacio v
		where a.codi_ciutada = Pcodi_ciutada
		and v.id_votacio = a.id_votacio		
		and a.id_votacio not in (select id_votacio from vota where codi_ciutada = Pcodi_ciutada);		
	BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '< ' ||Pcodi_ciutada|| ' >';
		-- Comprova parmetres d'entrada
        IF (Pcodi_ciutada IS NULL OR Pcodi_ciutada = 0) THEN
            RAISE noCiu;
        END IF;
		-- Comprova que existeixi el ciutad
        BEGIN 
            SELECT codi_ciutada INTO codi
			FROM Ciutada
            WHERE codi_ciutada = Pcodi_ciutada;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noCiu;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Llistat de votacions que ha participat o no un ciutad');
		DBMS_OUTPUT.PUT_LINE('------------------------------------------------------');
		DBMS_OUTPUT.PUT_LINE('Codi de ciutad: '||Pcodi_ciutada);
		DBMS_OUTPUT.PUT_LINE(' ');
		-- Llistem les votacions que ha participat el ciutad
		DBMS_OUTPUT.PUT_LINE('Votacions en les que ha participat');
		DBMS_OUTPUT.PUT_LINE('----------------------------------');
		BEGIN
			-- Tractament del cursor
			OPEN c_vots;
			LOOP
				FETCH c_vots INTO codivot, nomtitol;
				EXIT WHEN c_vots%NOTFOUND;
				DBMS_OUTPUT.PUT_LINE(nomtitol);
			END LOOP;
			CLOSE c_vots;
		END;
		-- Llistem les votacions que no ha participat el ciutad
		DBMS_OUTPUT.PUT_LINE(' ');		
		DBMS_OUTPUT.PUT_LINE('Votacions en les que no ha participat');
		DBMS_OUTPUT.PUT_LINE('-------------------------------------');
		BEGIN
			-- Tractament del cursor
			OPEN c_vots2;
			LOOP
				FETCH c_vots2 INTO codivot, nomtitol;
				EXIT WHEN c_vots2%NOTFOUND;
				DBMS_OUTPUT.PUT_LINE(nomtitol);
			END LOOP;
			CLOSE c_vots2;
		END;
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
		-- Control d'excepcions
        EXCEPTION
			WHEN noCodi THEN
				resultat := 'ERROR: <Ciutad> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCiu THEN
				resultat := 'ERROR: El ciutad no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_votacions_ciutada_llistat;
/
