-------------------------------------------------------------------------
-- Estadstiques - E1 - procediment d'actualitzaci 
--
-- Parmetres d'entrada: codi_pais, any_concret, total_vots
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
-------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e1_actualitza (
	Pcodi_pais IN Pais.codi_pais%TYPE,
	Pany_concret IN INTEGER,	
	Ptotal_vots IN INTEGER,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e1_actualitza';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noPais EXCEPTION;
	noAny EXCEPTION;
	noTotal EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	acumvots INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pcodi_pais || '> <' || Pany_concret || '> 
					<' || Ptotal_vots || '>';
        -- Comprovaci parmetres entrada
        IF (Pcodi_pais IS NULL OR Pcodi_pais = ' ') THEN
            RAISE noPais;
        END IF;
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
        IF (Ptotal_vots IS NULL OR Ptotal_vots = 0) THEN
            RAISE noTotal;
        END IF;
		-- Comprova que existeix un registre
        BEGIN 
            SELECT num_votacions INTO acumvots
			FROM E1
            WHERE any_votacio = Pany_concret
			AND codi_pais = Pcodi_pais;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
			-- Afegim nou registre ja que no existeix
			INSERT INTO e1 (codi_pais, any_votacio, num_votacions)
				VALUES (Pcodi_pais, Pany_concret, Ptotal_vots);
            RAISE faInsert;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Afegim els vots en el camp acumvots
		acumvots := acumvots + Ptotal_vots;
		-- Actualitzem el registre
        UPDATE e1 set 
			num_votacions = acumvots
        WHERE any_votacio = Pany_concret
		AND codi_pais = Pcodi_pais;

		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noPais THEN
				resultat := 'ERROR: <Codi pas> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTotal THEN
				resultat := 'ERROR: <Total vots> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E1 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e1_actualitza;
/


----------------------------------------------------------------------------
-- Estadstiques - E1 - Nombre de votacions finalitzades x pas i any
--
-- Parmetres d'entrada: id_votacio, any_concret
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert, p_e1_actualitza
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e1_estadistica (
	Pid_votacio IN votacio.id_votacio%TYPE,
	Pany_concret IN INTEGER,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e1_estadistica';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noVot EXCEPTION;
	noFinalitzada EXCEPTION;	
	noSeguir EXCEPTION;		
	altraExcepcio EXCEPTION;
	noTrobaVot EXCEPTION;	
	estatvot CHAR(1);
	codipais CHAR(2);	
	totalvots INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);
	RSP Varchar2(100);
	-- Definim el cursor
	CURSOR c_e1 IS
	-- Calcula el total de vots que ha tret cada pas 
		select p.codi_pais, count(v.codi_ciutada) as total 
		from vota v, ciutada c, localitat l, regio r, pais p
		where v.codi_ciutada = c.codi_ciutada
		and c.id_localitat = l.id_localitat
		and l.id_regio = r.id_regio
		and r.codi_pais = p.codi_pais
		and v.id_votacio = Pid_votacio
		group by p.codi_pais;
	BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
		RSP := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pany_concret || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
		-- Comprovaci de que la votacio existeixi
        BEGIN 
            SELECT estat INTO estatvot
			FROM Votacio
            WHERE Pid_votacio = id_votacio;
		IF (estatvot <> 'F') THEN RAISE noFinalitzada;
        END IF;		
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Bucle principal del cursor
		BEGIN
			-- Tractament del cursor
			OPEN c_e1;
			LOOP
				FETCH c_e1 INTO codipais, totalvots;
				EXIT WHEN c_e1%NOTFOUND;
				p_e1_actualitza(codipais, Pany_concret, totalvots, RSP);
				IF RSP <> 'OK' THEN 
					CLOSE c_e1;
					RAISE noSeguir;
				END IF;
			END LOOP;
			CLOSE c_e1;
		END;
		
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noFinalitzada THEN
				resultat := 'ERROR: La votaci no est finalitzada';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noSeguir THEN
				resultat := 'ERROR: Taula E1 mal actualitzada, mirar log';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e1_estadistica;
/
-------------------------------------------------------------------------
-- Estadstiques - E2 - procediment d'actualitzaci
--
-- Parmetres d'entrada: id_cens, any_concret, mitja
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
-------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e2_actualitza (
	Pid_cens IN Assignacio.id_cens%TYPE,
	Pany_concret IN INTEGER,	
	Pmitja IN INTEGER,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e2_actualitza';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noCens EXCEPTION;
	noAny EXCEPTION;
	noMitja EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	mitjana INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_cens || '> <' || Pany_concret || '> 
					<' || Pmitja || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCens;
        END IF;
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
        IF (Pmitja IS NULL OR Pmitja = 0) THEN
            RAISE noMitja;
        END IF;
		-- Comprova que existeix un registre
        BEGIN 
            SELECT valor_mitja INTO mitjana
			FROM E2
            WHERE any_votacio = Pany_concret
			AND id_cens = Pid_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
			-- Afegim nou registre ja que no existeix
			INSERT INTO e2 (id_cens, any_votacio, valor_mitja)
				VALUES (Pid_cens, Pany_concret, Pmitja);
            RAISE faInsert;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Actualitzem el registre
        UPDATE e2 set 
			valor_mitja = Pmitja
        WHERE any_votacio = Pany_concret
		AND id_cens = Pid_cens;
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
		-- Control d'excepcions
        EXCEPTION
			WHEN noCens THEN
				resultat := 'ERROR: <Codi cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noMitja THEN
				resultat := 'ERROR: <Mitja> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E2 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e2_actualitza;
/
----------------------------------------------------------------------------
-- Estadstiques - E2 - Valor mitj del percentatge de participaci
-- tenint en compte totes les votacions associades a aquell cens electoral
-- que han finalitzat durant aquell cens
--
-- Parmetres d'entrada: any_concret
-- Parmetres de sortida: RSP
-- Procediments que executa: p_log_insert, p_e2_actualitza
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e2_estadistica (
	Pany_concret IN INTEGER,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e2_estadistica';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noAny EXCEPTION;
	noFinalitzada EXCEPTION;	
	noSeguir EXCEPTION;		
	altraExcepcio EXCEPTION;
	noTrobaVot EXCEPTION;	
	estatvot CHAR(1);
	codicens CHAR(2);	
	totalmitja NUMBER(5,2);
	err_num NUMBER;
	err_msg Varchar2(255);
	RSP Varchar2(100);
	-- Definim el cursor
	CURSOR c_e2 IS
	-- Calcula la mitja del percentatge de participaci que ha tret cada cens x any 
		select a.id_cens, avg(v.perc_participacio) as mitja
		from assignacio a, votacio v
		where a.id_votacio = v.id_votacio
		and to_number(to_char(a.data_assignacio,'YYYY')) = Pany_concret
		and v.estat = 'F'
		group by a.id_cens
		order by mitja desc;
	BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
		RSP := 'OK';
        entrada := '<' || Pany_concret || '>';
        -- Comprovaci parmetres entrada
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
		-- Bucle principal del cursor
		BEGIN
			-- Tractament del cursor
			OPEN c_e2;
			LOOP
				FETCH c_e2 INTO codicens, totalmitja;
				EXIT WHEN c_e2%NOTFOUND;
				p_e2_actualitza(codicens, Pany_concret, totalmitja, RSP);
				IF RSP <> 'OK' THEN 
					CLOSE c_e2;
					RAISE noSeguir;
				END IF;
			END LOOP;
			CLOSE c_e2;
		END;
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
		-- Control d'excepcions
        EXCEPTION
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noFinalitzada THEN
				resultat := 'ERROR: La votaci no est finalitzada';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noSeguir THEN
				resultat := 'ERROR: Taula E2 mal actualitzada, mirar log';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e2_estadistica;
/

-------------------------------------------------------------------------
-- Estadstiques - E3 - procediment d'actualitzaci
--
-- Parmetres d'entrada: id_cens, any_concret, total_vots
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
-------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e3_actualitza (
	Pid_cens IN Assignacio.id_cens%TYPE,
	Pany_concret IN INTEGER,	
	Ptotal_vots IN INTEGER,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e3_actualitza';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noCens EXCEPTION;
	noAny EXCEPTION;
	noTotal EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	acumvots INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_cens || '> <' || Pany_concret || '> 
					<' || Ptotal_vots || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCens;
        END IF;
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
        IF (Ptotal_vots IS NULL OR Ptotal_vots = 0) THEN
            RAISE noTotal;
        END IF;
		-- Comprova que existeix un registre
        BEGIN 
            SELECT num_votacions INTO acumvots
			FROM E3
            WHERE any_votacio = Pany_concret
			AND id_cens = Pid_cens;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
			-- Afegim nou registre ja que no existeix
			INSERT INTO e3 (id_cens, any_votacio, num_votacions)
				VALUES (Pid_cens, Pany_concret, Ptotal_vots);
            RAISE faInsert;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Afegim els vots en el camp acumvots
		acumvots := acumvots + Ptotal_vots;
		-- Actualitzem el registre
        UPDATE e3 set 
			num_votacions = acumvots
        WHERE any_votacio = Pany_concret
		AND id_cens = Pid_cens;

		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCens THEN
				resultat := 'ERROR: <Codi cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTotal THEN
				resultat := 'ERROR: <Total vots> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E3 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e3_actualitza;
/
----------------------------------------------------------------------------
-- Estadstiques - E3 - Nombre de votacions associades a un cens x any
--
-- Parmetres d'entrada: id_votacio, any_concret
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert, p_e3_actualitza
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e3_estadistica (
	Pid_votacio IN votacio.id_votacio%TYPE,
	Pany_concret IN INTEGER,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e3_estadistica';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noVot EXCEPTION;
	noAny EXCEPTION;
	noFinalitzada EXCEPTION;	
	noSeguir EXCEPTION;		
	altraExcepcio EXCEPTION;
	noTrobaVot EXCEPTION;	
	estatvot CHAR(1);
	codicens CHAR(2);	
	totalassig INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);
	RSP Varchar2(100);
	-- Definim el cursor
	CURSOR c_e3 IS
	-- Calcula el total de vots que ha tret cada pas 
		select id_cens, count(*) as total
		from assignacio
		where to_number(to_char(data_assignacio,'YYYY')) = Pany_concret
		and id_votacio = Pid_votacio
		group by id_cens
		order by total desc;
	BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
		RSP := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pany_concret || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
		-- Comprovaci de que la votacio existeixi
        BEGIN 
            SELECT estat INTO estatvot
			FROM Votacio
            WHERE Pid_votacio = id_votacio;
		IF (estatvot <> 'F') THEN RAISE noFinalitzada;
        END IF;		
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTrobaVot;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		-- Bucle principal del cursor
		BEGIN
			-- Tractament del cursor
			OPEN c_e3;
			LOOP
				FETCH c_e3 INTO codicens, totalassig;
				EXIT WHEN c_e3%NOTFOUND;
				p_e3_actualitza(codicens, Pany_concret, totalassig, RSP);
				IF RSP <> 'OK' THEN 
					CLOSE c_e3;
					RAISE noSeguir;
				END IF;
			END LOOP;
			CLOSE c_e3;
		END;
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
		-- Control d'excepcions
        EXCEPTION
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noFinalitzada THEN
				resultat := 'ERROR: La votaci no est finalitzada';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTrobaVot THEN
				resultat := 'ERROR: La votaci no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noSeguir THEN
				resultat := 'ERROR: Taula E3 mal actualitzada, mirar log';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e3_estadistica;
/

----------------------------------------------------------------------------
-- Estadstiques - E4 - Votaci que ms participaci ha tingut
--
-- Parmetres d'entrada: id_votacio, any_concret, perc_participacio
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e4_estadistica (
	Pid_votacio IN Votacio.id_votacio%TYPE,
	Pany_concret IN INTEGER,	
	Pperc_participacio IN Votacio.perc_participacio%TYPE,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e4_estadistica';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noVot EXCEPTION;
	noAny EXCEPTION;
	noPerc EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	estatvot CHAR(1);
	percentatge NUMBER(5,2);
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pany_concret || '> 
					<' || Pperc_participacio || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
        IF (Pperc_participacio IS NULL OR Pperc_participacio = 0) THEN
            RAISE noPerc;
        END IF;
		-- Comprova que existeix un registre
        BEGIN 
            SELECT participacio_e4 INTO percentatge 
			FROM E4
            WHERE any_votacio = Pany_concret;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
			INSERT INTO E4 (any_votacio, id_votacio, participacio_e4)
				VALUES (Pany_concret, Pid_votacio, Pperc_participacio);
            RAISE faInsert;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		IF Pperc_participacio > percentatge THEN 
	        UPDATE e4 set 
			participacio_e4 = Pperc_participacio,
			id_votacio = Pid_votacio
			WHERE any_votacio = Pany_concret;
			RAISE faActualitzacio;
		END IF;
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPerc THEN
				resultat := 'ERROR: <Percentatge> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E4 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e4_estadistica;
/
----------------------------------------------------------------------------------
-- Estadstiques - E5 - Localitat europea que ms vots han ems els seus ciutadans
--
-- Parmetres d'entrada: any_concret
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e5_estadistica (
	Pany_concret IN INTEGER,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e5_estadistica';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noExisteix EXCEPTION;
	noAny EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	numvots INTEGER;
	totalvots INTEGER;
	localitat_top INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
	CURSOR c_e5 IS
	-- Calcula el total de votacions x localitat i any
		select c.id_localitat, count(*) as total
		from vota v, ciutada c
		where v.codi_ciutada = c.codi_ciutada
		and (to_number(to_char(data_hora_vot,'YYYY')) = Pany_concret)
		group by c.id_localitat
		order by total desc;
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pany_concret || '>'; 
        -- Comprovaci parmetres entrada
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
			-- Tractament del cursor per obtenir la localitat 
			-- i el nmero mxim de vots
		BEGIN
			totalvots := 0;
			localitat_top := 0;
			OPEN c_e5;
				FETCH c_e5 INTO localitat_top, totalvots;
			CLOSE c_e5;
		END;
		IF totalvots = 0 THEN RAISE noExisteix;
		END IF;
		-- Comprova que existeix un registre d'E5
        BEGIN 
            SELECT num_votacions_e5 INTO numvots
			FROM E5
            WHERE any_concret = Pany_concret;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
			INSERT INTO E5 (any_concret, id_localitat, num_votacions_e5)
				VALUES (Pany_concret, localitat_top, totalvots);
            RAISE faInsert;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		IF totalvots > numvots THEN 
	        UPDATE e5 set 
			num_votacions_e5 = totalvots,
			id_localitat = localitat_top
			WHERE any_concret = Pany_concret;
			RAISE faActualitzacio;
		END IF;
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
		-- Control d'excepcions
        EXCEPTION
			WHEN noExisteix THEN
				resultat := 'ERROR: No existeix cap registre en aquest any';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E5 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e5_estadistica;
/
----------------------------------------------------------------------------
-- Estadstiques - E6 - Percentatge de ciutadans que mai han votat
--
-- Parmetres d'entrada: any_concret
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e6_estadistica (
	Pany_concret IN INTEGER,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e6_estadistica';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noAny EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	total_no_voten INTEGER;
	total_si_voten INTEGER;	
	percentatge NUMBER(5,2);
	perc_total NUMBER(5,2);	
	codi INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
	CURSOR c_e6 IS
	-- Calcula el total de ciutadans que no han votat
		select distinct codi_ciutada
		from assignacio
		where codi_ciutada not in (select codi_ciutada from vota)
		and (to_number(to_char(data_assignacio,'YYYY')) = Pany_concret);
	CURSOR c_e6_bis IS
	-- Calcula el total de ciutadans que no han votat
		select distinct codi_ciutada
		from vota
		where (to_number(to_char(data_hora_vot,'YYYY')) = Pany_concret);
	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pany_concret || '>';
        -- Comprovaci parmetres entrada
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
		-- Tractament dels cursors per obtenir els totals amb el RowCount
		-- Total de no votants
		BEGIN
			-- Tractament del cursor
			total_no_voten := 0;
			OPEN c_e6;
			LOOP
				FETCH c_e6 INTO codi;
				EXIT WHEN c_e6%NOTFOUND;								
			END LOOP;
			total_no_voten := c_e6%ROWCOUNT;
			CLOSE c_e6;
		END;
		-- Total de votants
		BEGIN
			-- Tractament del cursor
			total_si_voten := 0;			
			OPEN c_e6_bis;
			LOOP
				FETCH c_e6_bis INTO codi;
				EXIT WHEN c_e6_bis%NOTFOUND;				
			END LOOP;
			total_si_voten := c_e6_bis%ROWCOUNT;
			CLOSE c_e6_bis;
		END;
		-- Clcul del percentatge
		IF total_si_voten = 0 THEN 
			perc_total := 100;
		ELSE
			perc_total := (total_no_voten * 100) / total_si_voten;
		END IF;
		-- Comprova que existeix un registre
        BEGIN 
            SELECT perc_ciutadans_no_votat INTO percentatge 
			FROM E6
            WHERE any_concret = Pany_concret;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
			INSERT INTO E6 (any_concret, perc_ciutadans_no_votat)
				VALUES (Pany_concret, perc_total);
            RAISE faInsert;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
        UPDATE e6 set 
			perc_ciutadans_no_votat = perc_total
		WHERE any_concret = Pany_concret;
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E6 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e6_estadistica;
/
----------------------------------------------------------------------------
-- Estadstiques - E7 - Votaci que histricament ha tingut ms participaci
--
-- Parmetres d'entrada: id_votacio, perc_participacio
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e7_estadistica (
	Pid_votacio IN Votacio.id_votacio%TYPE,
	Pperc_participacio IN Votacio.perc_participacio%TYPE,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e7_estadistica';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noVot EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	percentatge NUMBER(5,2);
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pperc_participacio || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
		-- Comprova que existeix un registre
        BEGIN 
            SELECT perc_participacio_e7 INTO percentatge 
			FROM E7
            WHERE id_e7 = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
			INSERT INTO E7 (id_votacio_participacio, perc_participacio_e7)
				VALUES (Pid_votacio, Pperc_participacio);
            RAISE faInsert;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		IF Pperc_participacio > percentatge THEN 
	        UPDATE e7 set 
			perc_participacio_e7 = Pperc_participacio,
			id_votacio_participacio = Pid_votacio
			WHERE id_e7 = 1;
			RAISE faActualitzacio;
		END IF;
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E7 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e7_estadistica;
/

----------------------------------------------------------------------------
-- Estadstiques - E8 - Votaci que histricament ha tingut ms diferncia
-- percentual entre l'opci ms votada i la menys
--
-- Parmetres d'entrada: id_votacio, diferencia
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e8_estadistica (
	Pid_votacio IN Votacio.id_votacio%TYPE,
	Pdiferencia IN Votacio.perc_participacio%TYPE,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e8_estadistica';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noVot EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	percentatge NUMBER(5,2);
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pdiferencia || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
		-- Comprova que existeix un registre
        BEGIN 
            SELECT diferencia_e8 INTO percentatge 
			FROM E8
            WHERE id_e8 = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
			INSERT INTO E8 (id_votacio_mes_dif, diferencia_e8)
				VALUES (Pid_votacio, Pdiferencia);
            RAISE faInsert;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		IF Pdiferencia > percentatge THEN 
	        UPDATE e8 set 
			diferencia_e8 = Pdiferencia,
			id_votacio_mes_dif = Pid_votacio
			WHERE id_e8 = 1;
			RAISE faActualitzacio;
		END IF;
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E8 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e8_estadistica;
/
----------------------------------------------------------------------------
-- Estadstiques - E9 - Votaci que histricament ha tingut menys diferncia
-- percentual entre l'opci ms votada i la menys
--
-- Parmetres d'entrada: id_votacio, diferencia
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e9_estadistica (
	Pid_votacio IN Votacio.id_votacio%TYPE,
	Pdiferencia IN Votacio.perc_participacio%TYPE,	
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e9_estadistica';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noVot EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	percentatge NUMBER(5,2);
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_votacio || '> <' || Pdiferencia || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_votacio IS NULL OR Pid_votacio = 0) THEN
            RAISE noVot;
        END IF;
		-- Comprova que existeix un registre
        BEGIN 
            SELECT diferencia_e9 INTO percentatge 
			FROM E9
            WHERE id_e9 = 1;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
			INSERT INTO E9 (id_votacio_menys_dif, diferencia_e9)
				VALUES (Pid_votacio, Pdiferencia);
            RAISE faInsert;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		IF Pdiferencia < percentatge THEN 
	        UPDATE e9 set 
			diferencia_e9 = Pdiferencia,
			id_votacio_menys_dif = Pid_votacio
			WHERE id_e9 = 1;
			RAISE faActualitzacio;
		END IF;
        
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noVot THEN
				resultat := 'ERROR: <Codi votaci> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E9 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e9_estadistica;
/
------------------------------------------------------------------------------
-- Estadstiques - E10 - Nmero mxim de votacions d'una persona i el seu pas
--
-- Parmetres d'entrada: 
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e10_estadistica (
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e10_estadistica';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noPais EXCEPTION;
	faInsert EXCEPTION;
	faActualitzacio EXCEPTION;	
	altraExcepcio EXCEPTION;
	percentatge NUMBER(5,2);
	primer INTEGER;
	codi INTEGER;	
	totalvots INTEGER;	
	maxim INTEGER;		
	codipais CHAR(2);
	err_num NUMBER;
	err_msg Varchar2(255);	
	-- Definim el cursor
	CURSOR c_e10 IS
	-- Calcula el total de vots que ha fet un ciutad 
		select codi_ciutada, count(*) as total
		from vota
		group by codi_ciutada
		order by total desc;
	BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '< Sense parmetres >';
		totalvots := 0;
		-- Esborrem tots els registres de la taula E10
		DELETE FROM E10;
		BEGIN
			-- Tractament del cursor
			OPEN c_e10;

			LOOP
				FETCH c_e10 INTO codi, totalvots;
				EXIT WHEN c_e10%NOTFOUND;		
				BEGIN
				-- Busquem el pas del ciutad	- obtenim el codi de pas
					select p.codi_pais into codipais
					from ciutada c, localitat l, regio r, pais p
					where c.codi_ciutada = codi
					and c.id_localitat = l.id_localitat
					and l.id_regio = r.id_regio
					and r.codi_pais = p.codi_pais;
				END;
				BEGIN
					INSERT INTO E10 (num_maxim_votacions, codi_pais_e10, codi_ciutada)
					VALUES (totalvots, codipais, codi);
				END;
			END LOOP;
			CLOSE c_e10;
		END;
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noPais THEN
				resultat := 'ERROR: El pas no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faInsert THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN faActualitzacio THEN
				-- Execuci correcta del procediment			
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
            WHEN DUP_VAL_ON_INDEX THEN
				resultat := 'ERROR: El registre E10 ja existeix';
				p_log_insert(procediment, entrada, sortida, resultat);						
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e10_estadistica;
/
