----------------------------------------------------------------------------
-- Llistats estadstics - p_e1_llistat  
-- 		Donat un pas i un any concret, el nombre de votacions
-- 		finalitzades que s'hi han produt
--
-- Parmetres d'entrada: codi_pais, any_votacio
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e1_llistat (
	Pcodi_pais IN Pais.codi_pais%TYPE,
	Pany_concret IN INTEGER,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e1_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noAny EXCEPTION;
	noPais EXCEPTION;	
	faInsert EXCEPTION;
	noPais2 EXCEPTION;	
    noTroba EXCEPTION;	
	altraExcepcio EXCEPTION;
	nomtitol Varchar2(100);
	nom Varchar2(100);
	total INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pcodi_pais || '> <' || Pany_concret || '>';
        -- Comprovaci parmetres entrada
        IF (Pcodi_pais IS NULL OR Pcodi_pais = ' ') THEN
            RAISE noPais;
        END IF;
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
		
		BEGIN
			DBMS_OUTPUT.PUT_LINE('Llistat E1 - Nombre de votacions per pas i any');
			DBMS_OUTPUT.PUT_LINE('-----------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
		END;
        BEGIN 
            SELECT p.nom_pais, num_votacions INTO nom, total
			FROM E1, Pais p
            WHERE e1.codi_pais = Pcodi_pais
			AND e1.codi_pais = p.codi_pais
			AND e1.any_votacio = Pany_concret;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTroba;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Codi de pas: '||Pcodi_pais);
		DBMS_OUTPUT.PUT_LINE('Pas: '||nom);
		DBMS_OUTPUT.PUT_LINE('Any de votaci: '||Pany_concret);		
		DBMS_OUTPUT.PUT_LINE('Nmero de votacions: '||total);		
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <Pas> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais2 THEN
				resultat := 'ERROR: El pas no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTroba THEN
				resultat := 'ERROR: El registre dE1 no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e1_llistat;
/
----------------------------------------------------------------------------
-- Llistats estadstics - p_e2_llistat 
-- 		Donat un cens i un any concret, el valor mitj del percentatge de 
-- 		participaci, tenint en compte totes les votacions associades
--		a aquell cens que han finalitzat aquell any
--
-- Parmetres d'entrada: cens, any_votacio
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e2_llistat (
	Pid_cens IN Cens.id_cens%TYPE,
	Pany_concret IN INTEGER,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e2_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noCens EXCEPTION;
	noPais EXCEPTION;	
	noAny EXCEPTION;	
	faInsert EXCEPTION;
	noPais2 EXCEPTION;	
    noTroba EXCEPTION;	
	altraExcepcio EXCEPTION;
	nom Varchar2(100);
	percentatge NUMBER(5,2);
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_cens || '> <' || Pany_concret || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCens;
        END IF;
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
	
		BEGIN
			DBMS_OUTPUT.PUT_LINE('Llistat E2 - Valor mitj percentatge participaci x cens i any');
			DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
		END;
        BEGIN 
            SELECT valor_mitja INTO percentatge
			FROM E2
            WHERE id_cens = Pid_cens
			AND any_votacio = Pany_concret;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTroba;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Codi de cens: '||Pid_cens);
		DBMS_OUTPUT.PUT_LINE('Any de votaci: '||Pany_concret);				
		DBMS_OUTPUT.PUT_LINE('Percentatge de participaci: '||percentatge||' %');		
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCens THEN
				resultat := 'ERROR: <Cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTroba THEN
				resultat := 'ERROR: El registre dE2 no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e2_llistat;
/

----------------------------------------------------------------------------
-- Llistats estadstics - p_e3_llistat 
-- 		Donat un cens i un any concret, el nombre de votacions que ha
-- 		tingut associades aquell cens durant aquell any
--
-- Parmetres d'entrada: cens, any_votacio
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e3_llistat (
	Pid_cens IN Cens.id_cens%TYPE,
	Pany_concret IN INTEGER,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e3_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noCens EXCEPTION;
	noPais EXCEPTION;	
	noAny EXCEPTION;	
	faInsert EXCEPTION;
	noPais2 EXCEPTION;	
    noTroba EXCEPTION;	
	altraExcepcio EXCEPTION;
	nom Varchar2(100);
	percentatge NUMBER(5,2);
	total INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid_cens || '> <' || Pany_concret || '>';
        -- Comprovaci parmetres entrada
        IF (Pid_cens IS NULL OR Pid_cens = 0) THEN
            RAISE noCens;
        END IF;
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
	
		BEGIN
			DBMS_OUTPUT.PUT_LINE('Llistat E3 - Nmero de votacions x cens i any');
			DBMS_OUTPUT.PUT_LINE('---------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
		END;
        BEGIN 
            SELECT num_votacions INTO total
			FROM E3
            WHERE id_cens = Pid_cens
			AND any_votacio = Pany_concret;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTroba;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Codi de cens: '||Pid_cens);
		DBMS_OUTPUT.PUT_LINE('Any de votaci: '||Pany_concret);				
		DBMS_OUTPUT.PUT_LINE('Nmero de votacions: '||total);		
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noCens THEN
				resultat := 'ERROR: <Cens> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTroba THEN
				resultat := 'ERROR: El registre dE3 no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e3_llistat;
/
----------------------------------------------------------------------------
-- Llistats estadstics - p_e4_llistat 
-- 		Donat un any concret, la votaci que ms participaci ha tingut
--
-- Parmetres d'entrada: any_concret
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e4_llistat (
	Pany_concret IN INTEGER,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e4_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noAny EXCEPTION;
    noTroba EXCEPTION;	
	altraExcepcio EXCEPTION;
	nomtitol Varchar2(100);
	percentatge INTEGER;
	codi INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pany_concret || '>';
        -- Comprovaci parmetres entrada
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
		BEGIN
			DBMS_OUTPUT.PUT_LINE('Llistat E4 - Votaci amb ms participaci x any');
			DBMS_OUTPUT.PUT_LINE('-----------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
		END;
        BEGIN 
            SELECT e4.id_votacio, e4.participacio_e4, v.titol INTO codi, percentatge, nomtitol
			FROM E4, Votacio v
            WHERE e4.any_votacio = Pany_concret
			AND e4.id_votacio = v.id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTroba;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Codi de votaci: '||codi);
		DBMS_OUTPUT.PUT_LINE('Ttol votaci: '||nomtitol);
		DBMS_OUTPUT.PUT_LINE('Any de votaci: '||Pany_concret);		
		DBMS_OUTPUT.PUT_LINE('Percentatge de participaci: '||percentatge||' %');		
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTroba THEN
				resultat := 'ERROR: El registre dE4 no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e4_llistat;
/
----------------------------------------------------------------------------
-- Llistats estadstics - p_e5_llistat 
-- 		Donat un any concret, la localitat europea que ms vots han ems
-- 		els seus ciutadans
--
-- Parmetres d'entrada: any_concret
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e5_llistat (
	Pany_concret IN INTEGER,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e5_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noAny EXCEPTION;
	noPais EXCEPTION;	
	faInsert EXCEPTION;
	noPais2 EXCEPTION;	
    noTroba EXCEPTION;	
	altraExcepcio EXCEPTION;
	nomtitol Varchar2(100);
	nom Varchar2(100);
	total INTEGER;
	codi INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pany_concret || '>';
        -- Comprovaci parmetres entrada
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
		BEGIN
			DBMS_OUTPUT.PUT_LINE('Llistat E5 - Localitat que ms vots han ems els seus ciutadans x any');
			DBMS_OUTPUT.PUT_LINE('---------------------------------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
		END;
        BEGIN 
            SELECT e5.id_localitat, l.nom_localitat, e5.num_votacions_e5 INTO codi, nom, total
			FROM E5, Localitat l
            WHERE e5.id_localitat = l.id_localitat
			AND e5.any_concret = Pany_concret;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTroba;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Codi de localitat: '||codi);
		DBMS_OUTPUT.PUT_LINE('Nom localitat: '||nom);
		DBMS_OUTPUT.PUT_LINE('Any de votaci: '||Pany_concret);		
		DBMS_OUTPUT.PUT_LINE('Nmero de votacions: '||total);		
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <Pas> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais2 THEN
				resultat := 'ERROR: El pas no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTroba THEN
				resultat := 'ERROR: El registre dE5 no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e5_llistat;
/
----------------------------------------------------------------------------
-- Llistats estadstics - p_e6_llistat 
-- 		Donat un any concret, percentatge de ciutadans que mai han votat
--
-- Parmetres d'entrada: any_concret
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e6_llistat (
	Pany_concret IN INTEGER,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e6_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noAny EXCEPTION;
	noPais EXCEPTION;	
	faInsert EXCEPTION;
	noPais2 EXCEPTION;	
    noTroba EXCEPTION;	
	altraExcepcio EXCEPTION;
	nomtitol Varchar2(100);
	nom Varchar2(100);
	percentatge NUMBER(5,2);
	codi INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pany_concret || '>';
        -- Comprovaci parmetres entrada
        IF (Pany_concret IS NULL OR Pany_concret = 0) THEN
            RAISE noAny;
        END IF;
		BEGIN
			DBMS_OUTPUT.PUT_LINE('Llistat E6 - Percentatge x any de ciutadans que mai han votat');
			DBMS_OUTPUT.PUT_LINE('-------------------------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
		END;
        BEGIN 
            SELECT perc_ciutadans_no_votat INTO percentatge
			FROM E6
            WHERE any_concret = Pany_concret;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTroba;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Any de votaci: '||Pany_concret);		
		DBMS_OUTPUT.PUT_LINE('Percentatge de ciutadans: '||percentatge||' %');		
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noAny THEN
				resultat := 'ERROR: <Any> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais THEN
				resultat := 'ERROR: <Pas> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noPais2 THEN
				resultat := 'ERROR: El pas no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTroba THEN
				resultat := 'ERROR: El registre dE6 no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e6_llistat;
/
----------------------------------------------------------------------------
-- Llistats estadstics - p_e7_llistat 
-- 		Votaci que histricament ha tingut ms participaci
--
-- Parmetres d'entrada: 1 --> que s l'nic registre de la taula E7
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e7_llistat (
	Pid IN INTEGER,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e7_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noId EXCEPTION;
	noPais EXCEPTION;	
	faInsert EXCEPTION;
	noPais2 EXCEPTION;	
    noTroba EXCEPTION;	
	altraExcepcio EXCEPTION;
	nomtitol Varchar2(100);
	nom Varchar2(100);
	percentatge NUMBER(5,2);
	codi INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid || '>';
        -- Comprovaci parmetres entrada
        IF (Pid IS NULL OR Pid = 0) THEN
            RAISE noId;
        END IF;
		BEGIN
			DBMS_OUTPUT.PUT_LINE('Llistat E7 - Votaci que histricament ha tingut ms participaci');
			DBMS_OUTPUT.PUT_LINE('-----------------------------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
		END;
        BEGIN 
            SELECT e7.id_votacio_participacio, v.titol, e7.perc_participacio_e7 
			INTO codi, nomtitol, percentatge
			FROM E7, Votacio v
            WHERE e7.id_E7 = Pid
			AND e7.id_votacio_participacio = v.id_votacio;			
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTroba;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Codi de votaci: '||codi);		
		DBMS_OUTPUT.PUT_LINE('Ttol votaci: '||nomtitol);		
		DBMS_OUTPUT.PUT_LINE('Percentatge participaci: '||percentatge||' %');		
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noId THEN
				resultat := 'ERROR: <Id> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTroba THEN
				resultat := 'ERROR: El registre dE7 no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e7_llistat;
/

----------------------------------------------------------------------------
-- Llistats estadstics - p_e8_llistat 
-- 		Votaci que histricament ha tingut ms diferncia percentual 
--		entre l'opci ms votada i la menys
--
-- Parmetres d'entrada: 1 --> que s l'nic registre de la taula E8
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e8_llistat (
	Pid IN INTEGER,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e8_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noId EXCEPTION;
	noPais EXCEPTION;	
	faInsert EXCEPTION;
	noPais2 EXCEPTION;	
    noTroba EXCEPTION;	
	altraExcepcio EXCEPTION;
	nomtitol Varchar2(100);
	nom Varchar2(100);
	percentatge NUMBER(5,2);
	codi INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid || '>';
        -- Comprovaci parmetres entrada
        IF (Pid IS NULL OR Pid = 0) THEN
            RAISE noId;
        END IF;
		BEGIN
			DBMS_OUTPUT.PUT_LINE('Llistat E8 - Votaci ms diferncia percentual entre opci ms votada i la menys');
			DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
		END;
        BEGIN 
            SELECT e8.id_votacio_mes_dif, v.titol, e8.diferencia_e8
			INTO codi, nomtitol, percentatge
			FROM E8, Votacio v
            WHERE e8.id_E8 = Pid
			AND e8.id_votacio_mes_dif = v.id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTroba;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Codi de votaci: '||codi);		
		DBMS_OUTPUT.PUT_LINE('Ttol votaci: '||nomtitol);		
		DBMS_OUTPUT.PUT_LINE('Diferncia percentual: '||percentatge||' %');		
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noId THEN
				resultat := 'ERROR: <Id> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTroba THEN
				resultat := 'ERROR: El registre dE8 no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e8_llistat;
/
----------------------------------------------------------------------------
-- Llistats estadstics - p_e9_llistat 
-- 		Votaci que histricament ha tingut menys diferncia percentual 
--		entre l'opci ms votada i la menys
--
-- Parmetres d'entrada: 1 --> que s l'nic registre de la taula E9
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e9_llistat (
	Pid IN INTEGER,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e9_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noId EXCEPTION;
	noPais EXCEPTION;	
	faInsert EXCEPTION;
	noPais2 EXCEPTION;	
    noTroba EXCEPTION;	
	altraExcepcio EXCEPTION;
	nomtitol Varchar2(100);
	nom Varchar2(100);
	percentatge NUMBER(5,2);
	codi INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pid || '>';
        -- Comprovaci parmetres entrada
        IF (Pid IS NULL OR Pid = 0) THEN
            RAISE noId;
        END IF;
		BEGIN
			DBMS_OUTPUT.PUT_LINE('Llistat E9 - Votaci menys diferncia percentual entre opci ms votada i la menys');
			DBMS_OUTPUT.PUT_LINE('----------------------------------------------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
		END;
        BEGIN 
            SELECT e9.id_votacio_menys_dif, v.titol, e9.diferencia_e9
			INTO codi, nomtitol, percentatge
			FROM E9, Votacio v
            WHERE e9.id_E9 = Pid
			AND e9.id_votacio_menys_dif = v.id_votacio;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTroba;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Codi de votaci: '||codi);		
		DBMS_OUTPUT.PUT_LINE('Ttol votaci: '||nomtitol);		
		DBMS_OUTPUT.PUT_LINE('Diferncia percentual: '||percentatge||' %');		
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noId THEN
				resultat := 'ERROR: <Id> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTroba THEN
				resultat := 'ERROR: El registre dE9 no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e9_llistat;
/
----------------------------------------------------------------------------
-- Llistats estadstics - p_e10_llistat 
-- 		Nmero mxim de votacions en les que ha participat una mateixa
--		persona i el pas d'aquesta persona, tenint en compte tota la
--		histria de votacions d'Europa
--
-- Parmetres d'entrada: codi_ciutada
-- Parmetres de sortida: resultat(RSP)
-- Procediments que executa: p_log_insert
----------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE p_e10_llistat (
	Pcodi_ciutada IN Ciutada.codi_ciutada%TYPE,
    resultat OUT NOCOPY Varchar2
	)
IS
    procediment Varchar2(80) := 'p_e10_llistat';
    entrada Varchar2(4000) := ' ';
	sortida Varchar2(4000) := ' ';
	noCiu EXCEPTION;
    noTroba EXCEPTION;	
	altraExcepcio EXCEPTION;
	nompais Varchar2(100);
	nom Varchar2(100);
	total INTEGER;
	codi INTEGER;
	err_num NUMBER;
	err_msg Varchar2(255);	
BEGIN
		-- Inicialitzaci de variables
		resultat := 'OK';
        entrada := '<' || Pcodi_ciutada || '>';
        -- Comprovaci parmetres entrada
        IF (Pcodi_ciutada IS NULL OR Pcodi_ciutada = 0) THEN
            RAISE noCiu;
        END IF;
		BEGIN
			DBMS_OUTPUT.PUT_LINE('Llistat E10 - Nmero mxim de votacions x ciutada');
			DBMS_OUTPUT.PUT_LINE('-------------------------------------------------');
			DBMS_OUTPUT.PUT_LINE(' ');
		END;
        BEGIN 
            SELECT e10.codi_ciutada, p.nom_pais, e10.num_maxim_votacions
			INTO codi, nompais, total
			FROM E10, Pais p
            WHERE e10.codi_ciutada = Pcodi_ciutada
			AND e10.codi_pais_e10 = p.codi_pais;
        EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RAISE noTroba;
        WHEN OTHERS THEN
            RAISE altraExcepcio;
        END;
		DBMS_OUTPUT.PUT_LINE('Codi ciutad: '||codi);		
		DBMS_OUTPUT.PUT_LINE('Pas: '||nompais);		
		DBMS_OUTPUT.PUT_LINE('Nmero de votacions: '||total);		
		-- Execuci correcta del procediment
        p_log_insert(procediment, entrada, sortida, resultat);
        
		-- Control d'excepcions
        EXCEPTION
			WHEN noCiu THEN
				resultat := 'ERROR: <Codi ciutad> inexistent';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN noTroba THEN
				resultat := 'ERROR: El registre dE9 no existeix';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN altraExcepcio THEN
				resultat := 'ERROR: Error indefinit';
				p_log_insert(procediment, entrada, sortida, resultat);
			WHEN OTHERS THEN
				err_num := SQLCODE;
				err_msg := SQLERRM;
				resultat := 'ERROR: ' || TO_CHAR(err_num) || '->' || err_msg ;
				p_log_insert(procediment, entrada, sortida, resultat);
END p_e10_llistat;
/
